@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -172,7 +172,8 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	static char msg[MAX_MESSAGE_SIZE];
 	line_desc *next_ld;
 	HIGHLIGHT_STATE next_line_state;
-	int error = OK, recording;
+	int error = OK;
+	char_stream *recording;
 	int64_t col;
 	char *q;
 
@@ -193,10 +194,12 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 	stop = false;
 
-	if (b->recording) record_action(b->cur_macro, a, c, p, verbose_macros);
+	if (recording_macro) record_action(recording_macro, a, c, p, verbose_macros);
 
 	if (perform_wrap > 0) perform_wrap--;
 
+	if (cur_buffer->visible_mark.shown) highlight_mark(cur_buffer, false);
+
 	switch(a) {
 
 	case EXIT_A:
@@ -316,20 +319,28 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case NEXTWORD_A:
-		NORMALIZE(c);
-		for(int64_t i = 0; i < c && !(error = search_word(b, 1)) && !stop; i++);
-		return stop ? STOPPED : error;
-
-	case PREVWORD_A:
-		NORMALIZE(c);
-		for(int64_t i = 0; i < c && !(error = search_word(b, -1)) && !stop; i++);
-		return stop ? STOPPED : error;
+	case PREVWORD_A: {
+			/* parse_word_parm(char *p, char *pat_in, int64_t *match) */
+			struct {
+				int64_t c;
+				int64_t left;
+				int64_t right;
+			} params = {-1, 0, 0};
+			error = parse_word_parm(p, "#<>", (int64_t *)&params) || (params.left && params.right);
+			if (!error) {
+				if (!(params.left || params.right)) params.left = 1;
+				NORMALIZE(params.c);
+				for(int64_t i = 0; i < params.c && !(error = search_word(b, a==NEXTWORD_A ? 1 : -1, params.left!=0)) && !stop; i++);
+			}
+			if (p) free(p);
+			return stop ? STOPPED : error;
+		}
 
 	case DELETENEXTWORD_A:
 	case DELETEPREVWORD_A:
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
-		recording = b->recording;
-		b->recording = 0;
+		recording = recording_macro;
+		recording_macro = NULL;
 		NORMALIZE(c);
 		int marking_t = b->marking;
 		int mark_is_vertical_t = b->mark_is_vertical;
@@ -349,11 +360,11 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 				that the cursor ends up here after an undo. */
 			insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, ' ');
 			delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, 1);
-			for(int64_t i = 0; i < c && !(error = search_word(b, -1)) && !stop; i++);
+			for(int64_t i = 0; i < c && !(error = search_word(b, -1, true)) && !stop; i++);
 		} else if (c > 0) {
 			move_to_eow(b);
 			if (b->block_start_line != b->cur_line || b->block_start_pos != b->cur_pos) c--;
-			for(int64_t i = 0; i < c && !(error = search_word(b, 1)) && !stop; i++);
+			for(int64_t i = 0; i < c && !(error = search_word(b, 1, true)) && !stop; i++);
 			if (c) move_to_eow(b);
 		}
 		const bool line_changed = b->block_start_line != b->cur_line;
@@ -380,11 +391,17 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		b->block_start_line = b->bookmark[WORDWRAP_BOOKMARK].line;
 		b->marking = marking_t;
 		b->mark_is_vertical = mark_is_vertical_t;
-		b->recording = recording;
+		recording_macro = recording;
 		return stop ? STOPPED : error;
 
 	case MOVEEOW_A:
-		move_to_eow(b);
+		if (!p || (p && !strcmp(p,">"))) move_to_eow(b);
+		else if (p && !strcmp(p,"<")) move_to_sow(b);
+		else if (p) {
+			free(p);
+			return SYNTAX_ERROR;
+		}
+		if (p) free(p);
 		return OK;
 
 	case MOVEINCUP_A:
@@ -396,9 +413,9 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case UNSETBOOKMARK_A:
-		if (p && p[0]=='*' && !p[1]) { /* Special parm "*" for UNSETBOOKMARK_A */
+		if (p && p[0] == '*' && !p[1]) { /* Special parm "*" for UNSETBOOKMARK_A */
 			b->bookmark_mask = b->cur_bookmark = 0;
-			print_message("All BookMarks cleared.");
+			print_message(info_msg[ALL_BOOKMARKS_CLEARED]);
 			free(p);
 			return OK;
 		} /* Intentionally fall through to regular BOOKMARK parm parsing. */
@@ -412,7 +429,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			if (p) {
 				if (p[0] == '?') {
 					free(p);
-					snprintf(msg, MAX_MESSAGE_SIZE, "Cur Bookmarks: [%s] %s (0-9, -1, +1, or '-')", cur_bookmarks_string(b), a==SETBOOKMARK_A?"SetBookmark":"GotoBookmark");
+					snprintf(msg, MAX_MESSAGE_SIZE, "Cur Bookmarks: [%s] %s (0-9, -1, +1, or '-')", cur_bookmarks_string(b), a == SETBOOKMARK_A ? "SetBookmark" : "GotoBookmark");
 					p = request_string(b, msg, NULL, true, COMPLETE_NONE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto);
 					if (!p) {
 						return INVALID_BOOKMARK_DESIGNATION;
@@ -421,17 +438,17 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			}
 			if (p) {
 				if ((p[0]=='+' || p[0]=='-') && p[1]=='1') {
-					if (b->cur_bookmark<0 || b->cur_bookmark>MAX_USER_BOOKMARK) b->cur_bookmark = 0;
+					if (b->cur_bookmark < 0 || b->cur_bookmark > MAX_USER_BOOKMARK) b->cur_bookmark = 0;
 					int i;
-					for(i = 0; i<=MAX_USER_BOOKMARK; i++) {
-						b->cur_bookmark = (b->cur_bookmark+MAX_USER_BOOKMARK+1+(p[0]=='+'?1:-1))%(MAX_USER_BOOKMARK+1);
-						if ((a==SETBOOKMARK_A?~b->bookmark_mask:b->bookmark_mask) & (1<<b->cur_bookmark)) {
+					for(i = 0; i <= MAX_USER_BOOKMARK; i++) {
+						b->cur_bookmark = (b->cur_bookmark + MAX_USER_BOOKMARK + 1 + (p[0] == '+' ? 1 : -1)) % (MAX_USER_BOOKMARK + 1);
+						if ((a == SETBOOKMARK_A ? ~b->bookmark_mask : b->bookmark_mask) & (1 << b->cur_bookmark)) {
 							c = b->cur_bookmark;
 							relative = true;
 							break;
 						}
 					}
-					if (i==MAX_USER_BOOKMARK+1) {
+					if (i==MAX_USER_BOOKMARK + 1) {
 						free(p);
 						switch (a) {
 						case SETBOOKMARK_A:
@@ -513,8 +530,8 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 	case INSERTSTRING_A:
 		/* Since we are going to call another action, we do not want to record this insertion twice. */
-		recording= b->recording;
-		b->recording = 0;
+		recording = recording_macro;
+		recording_macro = NULL;
 		error = ERROR;
 		if (p || (p = request_string(b, "String", NULL, false, COMPLETE_NONE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
 			encoding_type encoding = detect_encoding(p, strlen(p));
@@ -530,7 +547,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			end_undo_chain(b);
 			free(p);
 		}
-		b->recording = recording;
+		recording_macro = recording;
 		return error;
 
 	case TABS_A:
@@ -551,8 +568,8 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case INSERTTAB_A:
-		recording = b->recording;
-		b->recording = 0;
+		recording = recording_macro;
+		recording_macro = NULL;
 		NORMALIZE(c);
 		start_undo_chain(b);
 		if (b->opt.tabs) {
@@ -568,7 +585,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			}
 		}
 		end_undo_chain(b);
-		b->recording = recording;
+		recording_macro = recording;
 		return error;
 
 	case INSERTCHAR_A: {
@@ -896,14 +913,21 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		p = str_dup(b->filename);
 
 	case SAVEAS_A:
+		if (b->opt.read_only && !request_response(b, info_msg[SAVE_READ_ONLY_DOCUMENT],false)) {
+			free(p);
+			return DOCUMENT_NOT_SAVED;
+		}
 		if (p || (q = p = request_file(b, "Filename", b->filename))) {
 			print_info(SAVING);
 
-			if (buffer_file_modified(b, p) && !request_response(b, info_msg[a==SAVE_A ? FILE_HAS_BEEN_MODIFIED : FILE_ALREADY_EXISTS], false)) {
+			if (buffer_file_modified(b, p) && !request_response(b, info_msg[a == SAVE_A ? FILE_HAS_BEEN_MODIFIED : FILE_ALREADY_EXISTS], false)) {
 				free(p);
 				return DOCUMENT_NOT_SAVED;
 			}
+			c = b->opt.read_only;
+			SET_USER_FLAG(b, 0, opt.read_only);
 			error = save_buffer_to_file(b, p);
+			SET_USER_FLAG(b, c, opt.read_only);
 
 			if (!print_error(error)) {
 				const bool load_syntax = b->filename == NULL || ! same_str(extension(p), extension(b->filename));
@@ -942,51 +966,60 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		reset_window();
 		return OK;
 
-	case OPENNEW_A:
-		if (b = new_buffer()) {
-			reset_window();
-		} else {
-			if (p) free(p);
-			return OUT_OF_MEMORY;
-		}
-
 	case OPEN_A:
 		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], false)) return ERROR;
-
+	case OPENNEW_A:
 		if (p || (p = request_file(b, "Filename", b->filename))) {
 			static bool dprompt = false; /* Set to true if we ever respond 'yes' to the prompt. */
-
+			bool empty = is_buffer_empty(b);
+			if (b = new_buffer()) reset_window();
+			else {
+				if (p) free(p);
+				return OUT_OF_MEMORY;
+			}
 			buffer *dup = get_buffer_named(p);
-
 			/* 'c' -- flag meaning "Don't prompt if we've ever responded 'yes'." */
 			if (!dup || dup == b || (dprompt && !c) || (dprompt = request_response(b, info_msg[SAME_NAME], false))) {
 				error = load_file_in_buffer(b, p);
-				if (error != FILE_IS_MIGRATED
-					&& error != FILE_IS_DIRECTORY
-					&& error != IO_ERROR
-					&& error != FILE_IS_TOO_LARGE
-					&& error != OUT_OF_MEMORY
-					&& error != OUT_OF_MEMORY_DISK_FULL) {
+				if (error == FILE_DOES_NOT_EXIST && a == OPEN_A && (empty || request_response(b, info_msg[NO_SUCH_FILE_EXISTS], false)))
+					error = OK;
+				if (! error || (a == OPENNEW_A && error == FILE_DOES_NOT_EXIST)) { /* Keep the new buffer, or delete it? */
 					change_filename(b, p);
+					p = NULL;
 					b->syn = NULL; /* So that autoprefs will load the right syntax. */
 					if (b->opt.auto_prefs) {
 						if (b->allocated_chars - b->free_chars <= MAX_SYNTAX_SIZE) {
-							if (load_auto_prefs(b, NULL) == HAS_NO_EXTENSION)
-								load_auto_prefs(b, DEF_PREFS_NAME);
+							if (load_auto_prefs(b, NULL) == HAS_NO_EXTENSION) load_auto_prefs(b, DEF_PREFS_NAME);
 							reset_syntax_states(b);
 						}
 						else if (error == OK) error = FILE_TOO_LARGE_SYNTAX_HIGHLIGHTING_DISABLED;
 					}
+					buffer * old_buffer = (buffer *)cur_buffer->b_node.prev;
+					/* preserve cur_macro, find_string, and replace_string */
+
+					free_char_stream(cur_buffer->cur_macro);
+					cur_buffer->cur_macro = dup_stream(old_buffer->cur_macro);
+					cur_buffer->find_string = str_dup(old_buffer->find_string);
+					cur_buffer->replace_string = str_dup(old_buffer->replace_string);
+					cur_buffer->last_was_replace = old_buffer->last_was_replace;
+					cur_buffer->last_was_regexp = old_buffer->last_was_regexp;
+
+					if (a == OPEN_A) {
+						do_action(cur_buffer, PREVDOC_A, 1, NULL);
+						delete_buffer();
+					}
+				} else {
+					delete_buffer();
+					do_action(cur_buffer, PREVDOC_A, 1, NULL);
 				}
-				print_error(error);
-				reset_window();
-				return OK;
+			} else {
+				delete_buffer();
+				do_action(cur_buffer, PREVDOC_A, 1, NULL);
 			}
-			free(p);
-		}
-		if (a == OPENNEW_A) {
-			do_action(b, CLOSEDOC_A, 1, NULL);
-			do_action(cur_buffer, PREVDOC_A, 1, NULL);
+			print_error(error);
+			reset_window();
+			if (p) free(p);
+			return error;
 		}
 		return ERROR;
 
@@ -1095,10 +1128,9 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 							}
 
 							num_replace++;
-
-							if (last_replace_empty_match)
-								if (b->opt.search_back) error = char_left(cur_buffer);
-								else error = char_right(cur_buffer);
+							if (b->opt.search_back) error = char_left(b) ? NOT_FOUND : OK;
+							else if (last_replace_empty_match) error = char_right(cur_buffer) ? NOT_FOUND : OK;
+							if (error) break;
 						}
 
 						if (print_error(error)) {
@@ -1170,7 +1202,8 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		error = OK;
 		int64_t num_replace = 0;
 		start_undo_chain(b);
-		for (int64_t i = 0; i < c && ! stop && ! (error = (b->last_was_regexp ? find_regexp : find)(b, NULL, !b->last_was_replace, perform_wrap > 0)); i++)
+
+		for (int64_t i = 0; i < c && ! stop && ! (error = (b->last_was_regexp ? find_regexp : find)(b, NULL, !b->last_was_replace||((b->cur_pos + b->cur_line == 0)&&b->opt.search_back), perform_wrap > 0)); i++)
 			if (b->last_was_replace) {
 				const int64_t cur_char = b->cur_char;
 				const int cur_x = b->cur_x;
@@ -1188,9 +1221,9 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 					num_replace++;
 
-					if (last_replace_empty_match)
-						if (b->opt.search_back) error = char_left(cur_buffer);
-						else error = char_right(cur_buffer);
+					if (b->opt.search_back) error = char_left(b) ? NOT_FOUND : OK;
+					else if (last_replace_empty_match) error = char_right(cur_buffer) ? NOT_FOUND : OK;
+
 				}
 
 				if (error) break;
@@ -1402,21 +1435,38 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		}
 
 	case RECORD_A:
-		recording = b->recording;
-		SET_USER_FLAG(b, c, recording);
-		if (b->recording && !recording) {
-			b->cur_macro = reset_stream(b->cur_macro);
+		if (recording_macro) {
+			if (c < 0) {           /* normal completion */
+				free_char_stream(b->cur_macro);
+				b->cur_macro = recording_macro;
+				recording_macro = NULL;
+				print_message(info_msg[MACRO_RECORDING_COMPLETED]);
+			} else if (c == 0) {   /* cancel recording */
+				free_char_stream(recording_macro);
+				recording_macro = NULL;
+				print_message(info_msg[MACRO_RECORDING_CANCELLED]);
+			} else {
+				print_message(info_msg[INVALID_ARGUMENT_WHILE_RECORDING]);
+				return ERROR;
+			}
+		} else if (c == 1) {  /* resume recording */
+			if (recording_macro = dup_stream(b->cur_macro))
+				print_message(info_msg[MACRO_RECORD_APPENDING_STARTED]);
+		} else if (c < 0) {   /* start recording */
+			recording_macro = alloc_char_stream(0);
 			print_message(info_msg[STARTING_MACRO_RECORDING]);
+		} else {
+			print_message(info_msg[INVALID_ARGUMENT_WHILE_RECORDING]);
+			return ERROR;
 		}
-		else if (!b->recording && recording) print_message(info_msg[MACRO_RECORDING_COMPLETED]);
 		return OK;
 
 	case PLAY_A:
-		if (!b->recording && !b->executing_internal_macro) {
+		if (!recording_macro && !executing_macro) {
 			if (c < 0 && (c = request_number(b, "Times", 1))<=0) return NUMERIC_ERROR(c);
-			b->executing_internal_macro = 1;
-			for(int64_t i = 0; i < c && !(error = play_macro(b, b->cur_macro)); i++);
-			b->executing_internal_macro = 0;
+			executing_macro = 1;
+			for(int64_t i = 0; i < c && !(error = play_macro(b->cur_macro)); i++);
+			executing_macro = 0;
 			return print_error(error) ? ERROR : 0;
 		}
 		else return ERROR;
@@ -1468,10 +1518,11 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 
-		/* We always return ERROR after a buffer has been deleted. Otherwise,
-		   the calling routines (and macros) could work on an unexisting buffer. */
+		/* We used to return ERROR after a buffer has been deleted to prevent
+		   the calling routines (and macros) attempting to work on an nonexistant buffer,
+		   but that should no longer be an issue. */
 
-		return ERROR;
+		return OK;
 
 	case NEXTDOC_A: /* Was NEXT_BUFFER: */
 		if (b->b_node.next->next) cur_buffer = (buffer *)b->b_node.next;
@@ -1506,7 +1557,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		if (c < 0) c = 1;
 		SET_USER_FLAG(b, c, marking);
 		if (!b->marking) return(OK);
-		print_message(info_msg[a==MARK_A ? BLOCK_START_MARKED : VERTICAL_BLOCK_START_MARKED]);
+		print_message(info_msg[a == MARK_A ? BLOCK_START_MARKED : VERTICAL_BLOCK_START_MARKED]);
 		b->mark_is_vertical = (a == MARKVERT_A);
 		b->block_start_line = b->cur_line;
 		b->block_start_pos = b->cur_pos;
@@ -1712,7 +1763,9 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	case PARAGRAPH_A:
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
 		NORMALIZE(c);
-		for(int64_t i = 0; i < c && !(error = paragraph(b)) && !stop; i++);
+		start_undo_chain(b);
+		for(int64_t i = 0; i < c && !(error = paragraph(b, i == 0)) && !stop; i++);
+		end_undo_chain(b);
 		if (stop) error = STOPPED;
 		if (error == STOPPED) reset_window();
 		assert(b->cur_pos >= 0);
@@ -1775,7 +1828,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		if (b->atomic_undo) {
 			b->atomic_undo = 0;
 			while (b->link_undos) end_undo_chain(b);
-			print_message("AtomicUndo level: 0");
+			print_message(info_msg[ATOMIC_UNDO_LEVEL_0]);
 		}
 
 		for(int64_t i = 0; i < c && !(error = undo(b)) && !stop; i++);
@@ -1819,7 +1872,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		/* Since we are going to call other actions (INSERTSTRING_A and DELETEPREVWORD_A),
 		   we do not want to record this insertion twice. Also, we are counting on
 		   INSERTSTRING_A to handle character encoding issues. */
-		recording = b->recording;
+		recording = recording_macro;
 
 		int64_t pos = b->cur_pos;
 
@@ -1831,11 +1884,11 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 		int e;
 		if (p = autocomplete(p, msg, true, &e)) {
-			b->recording = 0;
+			recording_macro = NULL;
 			start_undo_chain(b);
 			if (pos >= b->cur_pos || (error = do_action(b, DELETEPREVWORD_A, 1, NULL)) == OK) error = do_action(b, INSERTSTRING_A, 0, p);
 			end_undo_chain(b);
-			b->recording = recording;
+			recording_macro = recording;
 			print_message(info_msg[e]);
 		}
 		else if (stop) error = STOPPED;
