@@ -1,6 +1,6 @@
 /* AutoComplete
 
-   Copyright (C) 2010-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 2010-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -31,33 +31,15 @@ static req_list rl;
 static int count_scanned;
 
 static void add_string(const char * const s, const int len, const int ext) {
-	static char *buf = NULL;
-	static uint64_t buflen = 0;
-	char *buf_new;
-	int cplen = len;
-
-	if (len < 1) {
-		if (buf) free(buf);
-		buf = NULL;
-		buflen = 0;
-		return;
-	}
-	if (len >= buflen) {
-		if (buf_new = realloc(buf, len * 2 + 1)) {
-			buflen = len * 2 + 1;
-			buf = buf_new;
-		}
-		else cplen = buflen - 1;
-	}
-	strncpy(buf, s, cplen);
-	buf[cplen] = '\0';
+	char *buf = strntmp(s, len);
+	if (len < 1) return;
 	req_list_add(&rl, buf, ext);
 }
 
 static void search_buff(const buffer *b, char * p, const int encoding, const bool case_search, const int ext) {
 	assert(p);
 	const int p_len = strlen(p);
-
+	const int (*cmp)(const char *, const char *, size_t) = (const int (*)(const char *, const char *, size_t))(case_search ? strncmp : strncasecmp);
 	for(line_desc *ld = (line_desc *)b->line_desc_list.head, *next; next = (line_desc *)ld->ld_node.next; ld = next) {
 		int64_t l = 0, r = 0;
 		do {
@@ -74,9 +56,10 @@ static void search_buff(const buffer *b, char * p, const int encoding, const boo
 				            || ( r+1 < ld->line_len && ch == '\'' && ne_isword(get_char(&ld->line[r+1], b->encoding), b->encoding))
 				          )
 				      ) r += get_char_width(&ld->line[r], b->encoding);
-				if (r - l > p_len && !(case_search ? strncmp : strncasecmp)(p, &ld->line[l], p_len)) {
-					if (b->encoding == encoding || is_ascii(&ld->line[l], r - l)) add_string(&ld->line[l], r - l, ext);
-				}
+				if ((b != cur_buffer || ld != b->cur_line_desc || b->cur_pos < l || r < b->cur_pos)
+				     && r - l > p_len && (b->encoding == encoding || is_ascii(&ld->line[l], r - l))
+				     && !cmp(p, &ld->line[l], p_len))
+					add_string(&ld->line[l], r - l, ext);
 				l = r;
 				count_scanned++;
 			}
@@ -101,10 +84,11 @@ static void search_buff(const buffer *b, char * p, const int encoding, const boo
 
 char *autocomplete(char *p, char *req_msg, const int ext, int * const error) {
 	int max_len = 0, min_len = INT_MAX, prefix_len = strlen(p);
-
+	static int ac_prune = true;
 	assert(p);
 
 	req_list_init(&rl, (cur_buffer->opt.case_search ? strcmp : strdictcmp), false, false, EXTERNAL_FLAG_CHAR);
+	rl.prune = ac_prune;
 	count_scanned = 0;
 
 	search_buff(cur_buffer, p, cur_buffer->encoding, cur_buffer->opt.case_search, false);
@@ -175,6 +159,7 @@ char *autocomplete(char *p, char *req_msg, const int ext, int * const error) {
 		else {
 			if (req_msg) print_message(req_msg);
 			int result = request_strings(&rl, 0);
+			ac_prune = rl.prune;
 			if (result != ERROR) {
 				result = result >= 0 ? result : -result - 2;
 				/* Delete EXTERNAL_FLAG_CHAR at the end of the strings if necessary. */
@@ -192,3 +177,5 @@ char *autocomplete(char *p, char *req_msg, const int ext, int * const error) {
 	D(fprintf(stderr, "autocomp returning '%s', entries: %d\n", p, rl.cur_entries);)
 	return p;
 }
+
+
