@@ -1,7 +1,7 @@
 /* Buffer handling functions, including allocation, deallocation, and I/O.
 
-   Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -19,7 +19,7 @@
    along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
-#include "ne.h" 
+#include "ne.h"
 #include "support.h"
 #include <sys/mman.h>
 
@@ -47,7 +47,7 @@ pool dimension, with respect to the number of lines of the given file. */
 
 /* The length of the block used in order to optimize saves. */
 
-#define SAVE_BLOCK_LEN	(16 * 1024 - 1)
+#define SAVE_BLOCK_LEN  (16 * 1024 - 1)
 
 /* The length of a half of the circular buffer used for memory mapping. */
 
@@ -147,7 +147,7 @@ line_desc_pool *alloc_line_desc_pool(int64_t pool_size, int force) {
 			ldp->mapped = force;
 			ldp->size = pool_size;
 			new_list(&ldp->free_list);
-			for(int64_t i = 0; i < pool_size; i++) 
+			for(int64_t i = 0; i < pool_size; i++)
 				if (do_syntax) add_tail(&ldp->free_list, &((line_desc *)ldp->pool)[i].ld_node);
 				else add_tail(&ldp->free_list, &((no_syntax_line_desc *)ldp->pool)[i].ld_node);
 			return ldp;
@@ -277,7 +277,7 @@ void free_buffer_contents(buffer * const b) {
 	b->filename = NULL;
 
 	reset_undo_buffer(&b->undo);
-	b->is_modified = b->marking = b->recording = b->x_wanted = 0;
+	b->is_modified = b->marking = b->x_wanted = 0;
 
 	release_signals();
 }
@@ -351,21 +351,57 @@ buffer *get_nth_buffer(int n) {
 
 
 
-/* Returns a buffer, given its name (i.e., the name of the file it
-   contains). Note that file_part() is applied *both* to the string passed
-   *and* to the buffer names, so that the path is immaterial. */
+/* Returns the first buffer with a name matching *p.
+   The buffers' names and *p are converted to their fully qualified form
+   for the comparisons.
+     This is a departure from earlier behavior, which only considered the
+   file_part() of either name. */
 
 buffer *get_buffer_named(const char *p) {
+	char *bname=NULL, *pname=NULL, *cwd=NULL;
+	buffer *b;
+	int rc = 1;
 	if (!p) return NULL;
-	p = file_part(p);
-
-	for(buffer *b = (buffer *)buffers.head; b->b_node.next; b = (buffer *)b->b_node.next)
-		if (b->filename && !strcmp(file_part(b->filename), p)) return b;
+	cwd = ne_getcwd(CUR_DIR_MAX_SIZE);
+	if (!cwd) return NULL;
 
+	if ((pname = absolute_file_path(p, cwd))) {
+		for(b = (buffer *)buffers.head; b->b_node.next; b = (buffer *)b->b_node.next) {
+			if (b->filename && (bname = absolute_file_path(b->filename, cwd))) {
+					if (!(rc = strcmp(bname, pname))) break;
+				}
+				free(bname);
+				bname = NULL;
+		}
+	}
+	free(pname);
+	free(bname);
+	free(cwd);
+	if (!rc) return b;
 	return NULL;
 }
 
 
+/* Returns true if the given pointer references an extant buffer. */
+
+bool is_buffer(const buffer * const maybe_buf) {
+	for(buffer *b = (buffer *)buffers.head; b->b_node.next; b = (buffer *)b->b_node.next)
+		if (maybe_buf == b) return true;
+	return false;
+}
+
+
+/* Returns true if the given buffer is empty. */
+
+bool is_buffer_empty(const buffer * const b) {
+	if (b) {
+		if (b->line_desc_list.head->next == NULL ||
+		    b->line_desc_list.head->next->next == NULL && ((line_desc *)b->line_desc_list.head)->line_len == 0) return true;
+	}
+	return false;
+}
+
+
 /* Returns true if any of the buffers has been modified since the last save. */
 
 int modified_buffers(void) {
@@ -407,7 +443,7 @@ int save_all_modified_buffers(void) {
 
 line_desc *alloc_line_desc(buffer * const b) {
 	block_signals();
-	
+
 	line_desc_pool *ldp;
 	for(ldp = (line_desc_pool *)b->line_desc_pool_list.head; ldp->ldp_node.next; ldp = (line_desc_pool *)ldp->ldp_node.next) {
 
@@ -465,7 +501,7 @@ void free_line_desc(buffer * const b, line_desc * const ld) {
 	for(ldp = (line_desc_pool *)b->line_desc_pool_list.head; ldp->ldp_node.next; ldp = (line_desc_pool *)ldp->ldp_node.next) {
 		assert_line_desc_pool(ldp);
 		if (do_syntax && ld >= (line_desc *)ldp->pool && ld < (line_desc *)ldp->pool + ldp->size
-			|| !do_syntax && (no_syntax_line_desc *)ld >= (no_syntax_line_desc *)ldp->pool 
+			|| !do_syntax && (no_syntax_line_desc *)ld >= (no_syntax_line_desc *)ldp->pool
 				&& (no_syntax_line_desc *)ld < (no_syntax_line_desc *)ldp->pool + ldp->size) break;
 	}
 
@@ -660,7 +696,7 @@ void free_chars(buffer *const b, char *const p, const int64_t len) {
    a stream containing one NULL. */
 
 
-int insert_one_line(buffer * const b, line_desc * const ld, const int64_t line, const int64_t pos) {	
+int insert_one_line(buffer * const b, line_desc * const ld, const int64_t line, const int64_t pos) {
 	return insert_stream(b, ld, line, pos, "", 1);
 }
 
@@ -694,7 +730,7 @@ int undelete_line(buffer * const b) {
 	line_desc * const ld = b->cur_line_desc;
 	if (!b->last_deleted) return ERROR;
 	start_undo_chain(b);
-	if (b->cur_pos > ld->line_len) 
+	if (b->cur_pos > ld->line_len)
 		insert_spaces(b, ld, b->cur_line, ld->line_len, b->win_x + b->cur_x - calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding));
 
 	insert_one_line(b, ld, b->cur_line, b->cur_pos);
@@ -718,7 +754,7 @@ void delete_to_eol(buffer * const b, line_desc * const ld, const int64_t line, c
 /* Inserts a stream in a line at a given position.  The position has to be
    smaller or equal to the line length. Since the stream can contain many
    lines, this function can be used for manipulating all insertions. It also
-   record the inverse operation in the undo buffer if b->opt.do_undo is
+   records the inverse operation in the undo buffer if b->opt.do_undo is
    true. */
 
 int insert_stream(buffer * const b, line_desc * ld, int64_t line, int64_t pos, const char * const stream, const int64_t stream_len) {
@@ -785,7 +821,7 @@ int insert_stream(buffer * const b, line_desc * ld, int64_t line, int64_t pos, c
 					if (len - result) memmove(ld->line - (len - result), ld->line, pos);
 					if (result) memmove(ld->line + pos + result, ld->line + pos, ld->line_len - pos);
 					memcpy(ld->line - (len - result) + pos, s, len);
-	
+
 					ld->line -= (len - result);
 					ld->line_len += len;
 				}
@@ -795,7 +831,7 @@ int insert_stream(buffer * const b, line_desc * ld, int64_t line, int64_t pos, c
 			/* We just inserted len chars at (line,pos); adjust bookmarks and mark accordingly. */
 			if (b->marking && b->block_start_line == line && b->block_start_pos > pos) b->block_start_pos += len;
 
-			for (int i = 0, mask = b->bookmark_mask; mask; i++, mask >>= 1) 
+			for (int i = 0, mask = b->bookmark_mask; mask; i++, mask >>= 1)
 				if ((mask & 1) && b->bookmark[i].line == line && b->bookmark[i].pos > pos) b->bookmark[i].pos += len;
 		}
 
@@ -824,7 +860,7 @@ int insert_stream(buffer * const b, line_desc * ld, int64_t line, int64_t pos, c
 				   adjust the buffer bookmarks and mark accordingly. */
 				if (b->marking) {
 					if (b->block_start_line == line && b->block_start_pos > pos) {
-						b->block_start_pos -= pos;
+						b->block_start_pos -= pos + len;
 						b->block_start_line++;
 					}
 					else if (b->block_start_line > line) b->block_start_line++;
@@ -832,7 +868,7 @@ int insert_stream(buffer * const b, line_desc * ld, int64_t line, int64_t pos, c
 				for (int i = 0, mask=b->bookmark_mask; mask; i++, mask >>= 1) {
 					if (mask & 1) {
 						if (b->bookmark[i].line == line && b->bookmark[i].pos > pos) {
-							b->bookmark[i].pos -= pos;
+							b->bookmark[i].pos -= pos + len;
 							b->bookmark[i].line++;
 						}
 						else if (b->bookmark[i].line > line) b->bookmark[i].line++;
@@ -869,7 +905,7 @@ int insert_one_char(buffer * const b, line_desc * const ld, const int64_t line,
 	assert(c != 0);
 
 	if (b->encoding == ENC_UTF8) t[utf8str(c, t)] = 0;
-	else t[0] = c,	t[1] = 0;
+	else t[0] = c, t[1] = 0;
 
 	return insert_stream(b, ld, line, pos, t, strlen(t));
 }
@@ -879,7 +915,7 @@ int insert_one_char(buffer * const b, line_desc * const ld, const int64_t line,
 /* Inserts a number of spaces. */
 
 int insert_spaces(buffer * const b, line_desc * const ld, const int64_t line, const int64_t pos, int64_t n) {
-	
+
 	static char spaces[MAX_STACK_SPACES];
 	int result = OK, i;
 
@@ -1076,7 +1112,7 @@ int delete_one_char(buffer * const b, line_desc * const ld, const int64_t line,
 	return delete_stream(b, ld, line, pos, b->encoding == ENC_UTF8 && pos < ld->line_len ? utf8len(ld->line[pos]) : 1);
 }
 
-/* Returns the line descriptor for line n of buffer b, or NULL if n is out of range. 
+/* Returns the line descriptor for line n of buffer b, or NULL if n is out of range.
    We assume that cur_line and cur_line_desc are coherent, and try to use the
    faster way (i.e., relative or absolute). */
 
@@ -1094,10 +1130,7 @@ line_desc *nth_line_desc(const buffer *b, const int64_t n) {
 		}
 		else {
 			ld = (line_desc *)b->line_desc_list.tail_pred;
-			for(int64_t i = 0; i < b->num_lines - 1 - n; i++) {
-				if ( i == -1 ) fputc('.', stderr); /* This is a nop that's here just to avoid a gcc bug. */
-				ld = (line_desc *)ld->ld_node.prev;
-			}
+			for(int64_t i = 0; i < b->num_lines - 1 - n; i++) ld = (line_desc *)ld->ld_node.prev;
 		}
 	}
 	else {
@@ -1105,7 +1138,7 @@ line_desc *nth_line_desc(const buffer *b, const int64_t n) {
 		if (n < b->cur_line) for(int64_t i = 0; i < b->cur_line - n; i++) ld = (line_desc *)ld->ld_node.prev;
 		else for(int64_t i = 0; i < n - b->cur_line; i++) ld = (line_desc *)ld->ld_node.next;
 	}
-	
+
 	return ld;
 }
 
@@ -1144,7 +1177,7 @@ int load_file_in_buffer(buffer * const b, const char *name) {
 		return result;
 	}
 
-	return CANT_OPEN_FILE;
+	return errno == ENOENT ? FILE_DOES_NOT_EXIST : CANT_OPEN_FILE;
 }
 
 /* Support function for load_fd_in_buffer(). */
@@ -1268,7 +1301,7 @@ int load_fd_mmap(buffer * const b, const int fd, const size_t len, char * const
 			(*ldp)->mapped = true;
 			b->allocated_chars = len;
 
-			/* We replace the offsets from the start of the file with actual memory 
+			/* We replace the offsets from the start of the file with actual memory
 			   pointers, while adding the line descriptors to the buffer list. */
 			if (do_syntax) {
 				for(line_desc *ld = (line_desc *)ld_p, *ld_end = ld + b->num_lines; ld < ld_end; ld++) {
@@ -1372,7 +1405,7 @@ int load_fd_in_buffer(buffer *b, int fd) {
 			pool = new_pool;
 			memset(pool + len, 0, curr_size - len);
 		}
-		
+
 		cp = alloc_char_pool_from_memory(pool, curr_size);
 		if (!cp) {
 			free(pool);
@@ -1488,7 +1521,7 @@ void reset_syntax_states(buffer *b) {
 		}
 
 		b->attr_len = -1;
-	}	
+	}
 }
 
 
@@ -1499,7 +1532,7 @@ void ensure_attr_buf(buffer * const b, const int64_t capacity) {
 	/* attr_buf already exists? */
 	if (!b->attr_buf) {
 		b->attr_size = capacity;
-		b->attr_buf = malloc(b->attr_size * sizeof *b->attr_buf); 
+		b->attr_buf = malloc(b->attr_size * sizeof *b->attr_buf);
 	}
 	else if (capacity > b->attr_size) {
 		b->attr_size = capacity;
@@ -1520,6 +1553,7 @@ int save_buffer_to_file(buffer *b, const char *name) {
 
 	assert_buffer(b);
 
+	if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
 	if (name == NULL) name = b->filename;
 
 	if (!name) return ERROR;
