@@ -4,7 +4,7 @@
 
 	Copyright (C) 1985, 1995 Free Software Foundation, Inc.
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -25,7 +25,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifndef TERMCAP
+#ifndef NE_TERMCAP
 #include <curses.h>
 #include <term.h>
 #else
