@@ -1,7 +1,7 @@
 /* Command table manipulation functions and vectors.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -28,12 +28,12 @@
 
 /* The standard macro descriptor allocation dimension. */
 
-#define STD_MACRO_DESC_SIZE	1024
+#define STD_MACRO_DESC_SIZE   1024
 
 
 /* This structure represents a command. It includes a long and a short name,
-a NULL-terminated vector of help strings (of specified length) and some flags
-which are related to the syntax and the semantics of the arguments. */
+   a NULL-terminated vector of help strings (of specified length) and some flags
+   which are related to the syntax and the semantics of the arguments. */
 
 typedef struct {
 	const char *name, *short_name;
@@ -122,7 +122,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(MOVEBOS       ), NO_ARGS                                                               },
 	{ NAHL(MOVEEOF       ), NO_ARGS                                                               },
 	{ NAHL(MOVEEOL       ), NO_ARGS                                                               },
-	{ NAHL(MOVEEOW       ),0                                                                      },
+	{ NAHL(MOVEEOW       ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(MOVEINCDOWN   ), NO_ARGS                                                               },
 	{ NAHL(MOVEINCUP     ), NO_ARGS                                                               },
 	{ NAHL(MOVELEFT      ),0                                                                      },
@@ -132,9 +132,9 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(MOVETOS       ), NO_ARGS                                                               },
 	{ NAHL(NAMECONVERT   ),                           IS_OPTION                                   },
 	{ NAHL(NEWDOC        ), NO_ARGS                                                               },
-	{ NAHL(NEXTDOC       ),0                                                                      },
+	{ NAHL(NEXTDOC       ), NO_ARGS                                                               },
 	{ NAHL(NEXTPAGE      ),0                                                                      },
-	{ NAHL(NEXTWORD      ),0                                                                      },
+	{ NAHL(NEXTWORD      ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(NOFILEREQ     ),                           IS_OPTION                                   },
 	{ NAHL(NOP           ), NO_ARGS                                                               },
 	{ NAHL(OPEN          ),           ARG_IS_STRING                                               },
@@ -149,11 +149,11 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(PLAY          ),0                                                                      },
 	{ NAHL(POPPREFS      ),0                                                                      },
 	{ NAHL(PRESERVECR    ),                           IS_OPTION                                   },
-	{ NAHL(PREVDOC       ),0                                                                      },
+	{ NAHL(PREVDOC       ), NO_ARGS                                                               },
 	{ NAHL(PREVPAGE      ),0                                                                      },
-	{ NAHL(PREVWORD      ),0                                                                      },
+	{ NAHL(PREVWORD      ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(PUSHPREFS     ),                           IS_OPTION                                   },
-	{ NAHL(QUIT          ), NO_ARGS                                                               },
+	{ NAHL(QUIT          ),                                       DO_NOT_RECORD                   },
 	{ NAHL(READONLY      ),                           IS_OPTION                                   },
 	{ NAHL(RECORD        ),                           IS_OPTION | DO_NOT_RECORD                   },
 	{ NAHL(REDO          ),0                                                                      },
@@ -199,7 +199,6 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(VERBOSEMACROS ),                           IS_OPTION                                   },
 	{ NAHL(VISUALBELL    ),                           IS_OPTION                                   },
 	{ NAHL(WORDWRAP      ),                           IS_OPTION                                   },
-
 };
 
 
@@ -219,9 +218,6 @@ int cmdcmp(const char *c, const char *m) {
 }
 
 
-
-
-
 /* This table *can* have conflicts, so that we can keep its size much
    smaller. */
 
@@ -236,7 +232,6 @@ static macro_desc *macro_hash_table[MACRO_HASH_TABLE_SIZE];
    this doesn't seem to produce any improvement. hash_macro() act as hash(),
    but uses MACRO_HASH_TABLE_SIZE for its modulo. */
 
-
 static int hash_cmd(const char * const s, int len) {
 	int h = -1;
 	while(len-- != 0) h = (h * 31 + ascii_up_case[(unsigned char)s[len]]) % HASH_TABLE_SIZE;
@@ -251,7 +246,6 @@ static int hash_macro(const char * const s, int len) {
 }
 
 
-
 /* Parses a command line. This function has an interface which is slightly
    varied with respect to the other functions of ne. In case of a parsing
    error, an error index *with sign inverted* is passed back. In case parsing
@@ -349,7 +343,6 @@ int execute_command_line(buffer *b, const char *command_line) {
 }
 
 
-
 /* Allocates a macro descriptor. It does not allocate the internal character
    stream, which has to be allocated and stuffed in separately. */
 
@@ -358,8 +351,7 @@ macro_desc *alloc_macro_desc(void) {
 }
 
 
-
-/* Frees a macro descriptors. */
+/* Frees a macro descriptor. */
 
 void free_macro_desc(macro_desc *md) {
 
@@ -385,7 +377,7 @@ void record_action(char_stream *cs, action a, int64_t c, const char *p, bool ver
 	char t[MAX_INT_LEN + 2];
 
 	/* NOP_A is special; it may actually be a comment.
-		Blank lines and real NOPs are recorded as blank lines. */
+	   Blank lines and real NOPs are recorded as blank lines. */
 	if (a == NOP_A) {
 		if (p && *p) add_to_stream(cs, p, strlen(p) + 1);
 		else add_to_stream(cs, "", 1);
@@ -428,7 +420,7 @@ static int insertchar_val(const char *p) {
 
 	action a;
 	if (((a = hash_table[h]) && !cmdcmp(commands[--a].name, cmd)
-		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, cmd))	&& a == INSERTCHAR_A) {
+		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, cmd)) && a == INSERTCHAR_A) {
 
 		while(isasciispace(*p)) p++;
 		h = strtol(p, (char **)&cmd, 0);
@@ -437,8 +429,9 @@ static int insertchar_val(const char *p) {
 	return 0;
 }
 
-/* Optimizing macros is not safe if there are any subsequent undo commands
-   calls to macros (which may themselves contain undo commands). This function
+
+/* Optimizing macros is not safe if there are any subsequent undo commands or
+   calls to other macros (which may themselves contain undo commands). This function
    looks through a stream for undo or non-built in commands, and returns false
    if any are found; returns true otherwise. */
 
@@ -460,6 +453,7 @@ bool vet_optimize_macro_stream(char_stream * const cs, int64_t pos) {
 	return true;
 }
 
+
 /* Looks through the macro stream for consecutive runs of InsertChar commands
    and replaces them with appropriate InsertString commands. This makes macros
    much easier to read if and when they have to be edited. Note that if the
@@ -497,14 +491,13 @@ void optimize_macro(char_stream *cs, bool verbose) {
 }
 
 
-/* This function is the ultimate goal of this file. It plays a character
-   stream, considering each line as a command line. It polls the global stop
-   variable in order to check for the user interrupting. Note that the macro is
-   duplicated before execution: this is absolutely necessary, for otherwise a
-   call to CloseDoc, Record or UnloadMacros could free() the block of memory
-   which we are executing. */
+/* Plays a character stream, considering each line as a command line. It
+   polls the global stop variable in order to check for the user interrupting.
+   Note that the macro is duplicated before execution: this is absolutely
+   necessary, for otherwise a call to CloseDoc, Record or UnloadMacros could
+   free() the block of memory which we are executing. */
 
-int play_macro(buffer *b, char_stream *cs) {
+int play_macro(char_stream *cs) {
 	if (!cs) return ERROR;
 
 	/* If len is 0 or 1, the character stream does not contain anything. */
@@ -519,14 +512,13 @@ int play_macro(buffer *b, char_stream *cs) {
 
 	stop = false;
 
-	b->executing_macro = 1;
 	int error = OK;
-	while(!stop && p - stream < len) {	
+	while(!stop && p - stream < len) {
 #ifdef NE_TEST
 		fprintf(stderr, "%s\n", p); /* During tests, we output to stderr the current command. */
 #endif
 
-		if (error = execute_command_line(b, p))
+		if (error = execute_command_line(cur_buffer, p))
 #ifndef NE_TEST
 			break /* During tests, we never interrupt a macro. */
 #endif
@@ -545,7 +537,6 @@ int play_macro(buffer *b, char_stream *cs) {
 }
 
 
-
 /* Loads a macro, and puts it in the global macro hash table.  file_part is
    applied to the name argument before storing it and hashing it.  Note that if
    the macro can't be opened, we retry prefixing its name with the preferences
@@ -553,7 +544,6 @@ int play_macro(buffer *b, char_stream *cs) {
    name does not conflict with internal commands can be executed transparently
    just by typing their name. */
 
-
 macro_desc *load_macro(const char *name) {
 
 	assert(name != NULL);
@@ -599,9 +589,8 @@ macro_desc *load_macro(const char *name) {
 }
 
 
-
 /* Executes a named macro. If the macro is not in the global
-macro list, it is loaded. A standard error code is returned. */
+   macro list, it is loaded. A standard error code is returned. */
 
 int execute_macro(buffer *b, const char *name) {
 	static int call_depth = 0;
@@ -621,14 +610,14 @@ int execute_macro(buffer *b, const char *name) {
 	assert_macro_desc(md);
 
 	if (md) {
-		if (b->recording) {
-			add_to_stream(b->cur_macro, "# include macro ", 16);
-			add_to_stream(b->cur_macro, md->name, strlen(md->name)+1);
+		if (recording_macro) {
+			add_to_stream(recording_macro, "# include macro ", 16);
+			add_to_stream(recording_macro, md->name, strlen(md->name)+1);
 		}
-		h = play_macro(b, md->cs);
-		if (b->recording) {
-			add_to_stream(b->cur_macro, "# conclude macro ", 17);
-			add_to_stream(b->cur_macro, md->name, strlen(md->name)+1);
+		h = play_macro(md->cs);
+		if (recording_macro) {
+			add_to_stream(recording_macro, "# conclude macro ", 17);
+			add_to_stream(recording_macro, md->name, strlen(md->name)+1);
 		}
 		--call_depth;
 		return h;
@@ -655,19 +644,20 @@ void unload_macros(void) {
 
 /* Find first n key strokes that currently map to commands[i].name or commands[i].short_name.
    Returns either NULL or a char string that must be freed by the caller. */
+
 char *find_key_strokes(int c, int n) {
 	char *str=NULL, *p;
 
 	for(int i = 0; i < NUM_KEYS && n; i++) {
 		if (key_binding[i]) {
 			if (((!strncasecmp(commands[c].short_name, key_binding[i], strlen(commands[c].short_name))) &&
-				  ((!key_binding[i][strlen(commands[c].short_name)]		) || 
-					(key_binding[i][strlen(commands[c].short_name)] == ' ')
+				  ((!key_binding[i][strlen(commands[c].short_name)]      ) ||
+				   (key_binding[i][strlen(commands[c].short_name)] == ' ')
 				  )
-				 ) || 
+				 ) ||
 				 ((!strncasecmp(commands[c].name, key_binding[i], strlen(commands[c].name))) &&
-				  ((!key_binding[i][strlen(commands[c].name)]		) ||
-					(key_binding[i][strlen(commands[c].name)] == ' ')
+				  ((!key_binding[i][strlen(commands[c].name)]      ) ||
+				   (key_binding[i][strlen(commands[c].name)] == ' ')
 				  )
 				 )
 				) {
@@ -688,6 +678,7 @@ char *find_key_strokes(int c, int n) {
 	return str;
 }
 
+
 char *bound_keys_string(int c) {
 	char *key_strokes = find_key_strokes(c, 9);
 	char *str=NULL;
@@ -698,25 +689,36 @@ char *bound_keys_string(int c) {
 	}
 	return str;
 }
-	
-	
+
+
 /* This function helps. The help text relative to the command name pointed to
    by p is displayed (p can also contain arguments). The string *p is not
-   free'd by help(). If p is NULL, the
-   alphabetically ordered list of commands is displayed with the string
-   requester. The help finishes when the user escapes. */
+   free'd by help(). If p is NULL, the alphabetically ordered list of commands
+   is displayed with the string requester. The help finishes when the user
+   escapes.
+
+   WARNING: help() assumes a lot about how request_strings() and 'req_list's
+   work rather than using the support functions to build its req_list. Any
+   changes here or in request.c need to be thoroughly checked in both places. */
 
 void help(char *p) {
-	req_list rl = { .ignore_tab=true };
+	bool request_order_orig = req_order;
+	req_list rl = { .ignore_tab=true, .help_quits=true };
 
 	D(fprintf(stderr, "Help Called with parm %p.\n", p);)
-	int r = 0;
+	int r = 0, width;
 	do {
 		print_message(info_msg[HELP_KEYS]);
 		rl.cur_entries = ACTION_COUNT;
 		rl.alloc_entries = 0;
-		rl.max_entry_len = MAX_COMMAND_WIDTH;
 		rl.entries = (char **)command_names;
+		rl.lengths = realloc(rl.lengths, sizeof(int) * rl.cur_entries);
+		width = 0;
+		for (int i=0,w; i<rl.cur_entries; i++)
+			if ((w=strlen(rl.entries[i])) > width) width = w;
+		for (int i=0; i<rl.cur_entries; i++)
+			rl.lengths[i] = width + 2;
+		req_order = request_order_orig;
 		if (p || (r = request_strings(&rl, r)) >= 0) {
 			D(fprintf(stderr, "Help check #2: p=%p, r=%d\n", p, r);)
 			if (p) {
@@ -752,24 +754,73 @@ void help(char *p) {
 				tmphelp[0] = (char *)commands[r].help[0];
 				tmphelp[1] = (char *)commands[r].help[1];
 				tmphelp[2] = key_strokes;
-				memcpy(&tmphelp[3], &commands[r].help[2], sizeof(char *) * (commands[r].help_len-2)); 
+				memcpy(&tmphelp[3], &commands[r].help[2], sizeof(char *) * (commands[r].help_len-2));
 				rl.cur_entries = commands[r].help_len+1;
 				rl.alloc_entries = 0;
-				rl.max_entry_len = ne_columns;
 				rl.entries = tmphelp;
+				rl.lengths = realloc(rl.lengths, sizeof(int) * rl.cur_entries);
+				width = 0;
+				for (int i=0,w; i<rl.cur_entries; i++)
+					if ((w=strlen(rl.entries[i])) > width) width = w;
+				for (int i=0; i<rl.cur_entries; i++)
+					rl.lengths[i] = width + 2;
+				req_order = true;
 				const int s = request_strings(&rl, 0);
+				req_order = request_order_orig;
 				if (s < 0) r = s;
 				free(tmphelp);
 			} else {
 				rl.cur_entries = commands[r].help_len;
 				rl.alloc_entries = 0;
-				rl.max_entry_len = ne_columns;
 				rl.entries = (char **)commands[r].help;
+				rl.lengths = realloc(rl.lengths, sizeof(int) * rl.cur_entries);
+				width = 0;
+				for (int i=0,w; i<rl.cur_entries; i++)
+					if ((w=strlen(rl.entries[i])) > width) width = w;
+				for (int i=0; i<rl.cur_entries; i++)
+					rl.lengths[i] = width + 2;
+				req_order = true;
 				const int s = request_strings(&rl, 0);
+				req_order = request_order_orig;
 				if (s < 0) r = s;
 			}
 			if (key_strokes) free(key_strokes);
 		}
 	} while(r >= 0);
+	free(rl.lengths);
 	draw_status_bar();
 }
+
+
+/* Parse string parameters for NextWord, PrevWord, AdjustView, etc. */
+
+int parse_word_parm(char *p, char *pat_in, int64_t *match) {
+	int i, len = strlen(pat_in);
+	char *pat = strntmp(pat_in, len);
+	if (p) {
+		while (*p) {
+			if (isasciispace(*p)) p++;
+			else if (isdigit((unsigned char)*p)) {
+				for (i=0; i<len; i++) {
+					if (pat[i] == '#') {
+						errno = 0;
+						match[i] = strtoll(p, &p, 10);
+						if (errno) return ERROR;
+						pat[i] = '\0';
+						break;
+					}
+				}
+				if (i==len) return ERROR;
+			} else {
+				for (i=0; i<len; i++) {
+					if (toupper(pat[i]) == toupper(*p)) {
+						match[i] = *p++;
+						break;
+					}
+				}
+				if (i==len) return ERROR;
+			}
+		}
+	}
+	return OK;
+}
