@@ -1,7 +1,7 @@
 /* Various editing functions such as word wrap, to upper, etc.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -39,7 +39,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 	int c;
 	/* First of all, we search for the word start, if we're not over it. */
 	if (pos >= b->cur_line_desc->line_len || !ne_isword(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding))
-		if (search_word(b, 1) != OK)
+		if (search_word(b, 1, true) != OK)
 			return ERROR;
 
 	bool changed = false;
@@ -86,6 +86,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 		start_undo_chain(b);
 
 		delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, len);
+
 		if (new_len) insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, word, new_len);
 
 		free(word);
@@ -97,7 +98,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 		need_attr_update = true;
 	}
 
-	return search_word(b, 1);
+	return search_word(b, 1, true);
 }
 
 
@@ -276,7 +277,7 @@ int word_wrap2(buffer * const b) {
 	b->bookmark[WORDWRAP_BOOKMARK].line  = original_line = b->cur_line;
 	b->bookmark[WORDWRAP_BOOKMARK].cur_y = b->cur_y;
 	b->bookmark_mask |= (1 << WORDWRAP_BOOKMARK);
-	paragraph(b);
+	paragraph(b, false);
 	goto_line_pos(b, b->bookmark[WORDWRAP_BOOKMARK].line, b->bookmark[WORDWRAP_BOOKMARK].pos);
 	line = b->cur_line_desc->line;
 	b->bookmark[WORDWRAP_BOOKMARK].cur_y += b->bookmark[WORDWRAP_BOOKMARK].line - original_line;
@@ -377,9 +378,15 @@ static int is_part_of_paragraph(const line_desc * const ld, const int tab_size,
      * on the last line of the paragraph.
 
    paragraph() returns OK unless the cursor ends up on the last line of the
-   file, in which case it returns ERROR. */
+   file, in which case it returns ERROR. It may return STOPPED if the user
+   interrupts a long-running operation.
 
-int paragraph(buffer * const b) {
+   mark_for_undo indicates whether to do an otherwise useless insertion and
+   deletion of a single character to ensure the cursor ends up at its initial
+   position after an undo. If performing a series of paragraph() calls within
+   an undo chain, it's only useful to do this on the first call. */
+
+int paragraph(buffer * const b, const bool mark_for_undo) {
 	line_desc *ld = b->cur_line_desc, *start_line_desc = ld;
 
 	if (!ld->line) return line_down(b);
@@ -395,13 +402,13 @@ int paragraph(buffer * const b) {
 	int64_t pos = b->cur_pos;
 	b->cur_pos = -1;
 
-	start_undo_chain(b);
-
-	/* This insertion and deletion of a single character ensures
-	   that the cursor ends up here after an undo. */
 	int64_t line = b->cur_line;
-	insert_one_char(b, ld, line, 0, ' ');
-	delete_stream(b, ld, line, 0, 1);
+	if (mark_for_undo) {
+		/* This insertion and deletion of a single character ensures
+		   that the cursor ends up here after an undo. */
+		insert_one_char(b, ld, line, 0, ' ');
+		delete_stream(b, ld, line, 0, 1);
+	}
 
 	const int right_margin = b->opt.right_margin ? b->opt.right_margin : ne_columns;
 	bool done;
@@ -469,8 +476,6 @@ int paragraph(buffer * const b) {
 		}
 	} while (!stop && !done);
 
-	end_undo_chain(b);
-
 	if (pa_space) {
 		free(pa_space);
 		pa_space = NULL;
@@ -483,7 +488,7 @@ int paragraph(buffer * const b) {
 	}
 	update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
 
-	goto_line_pos(b, line, pos);
+	goto_line_pos(b, line, ld->line_len);
 	if (stop || line_down(b) == ERROR) return stop ? STOPPED : ERROR;
 
 	/* Try to find the first non-blank starting with this line. */
@@ -491,8 +496,9 @@ int paragraph(buffer * const b) {
 	line = b->cur_line;
 
 	do {
+		pos = 0;
 		if (ld->line) {
-			for (pos = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding)) {
+			for (; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding)) {
 				if (!isasciispace(ld->line[pos])) {
 					goto_line_pos(b, line, pos);
 					return ld->ld_node.next ? OK : ERROR;
@@ -502,8 +508,8 @@ int paragraph(buffer * const b) {
 		ld = (line_desc *)ld->ld_node.next;
 		line++;
 	} while (ld->ld_node.next);
-
-	return b->cur_line_desc->ld_node.next ? OK : ERROR;
+	goto_line_pos(b, line - 1, pos);
+	return ERROR;
 }
 
 
