@@ -1,7 +1,7 @@
 /* Error message vector.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -54,56 +54,60 @@ char *error_msg[ERROR_COUNT] = {
 	/* 26 */ "Command has no arguments.",
 	/* 27 */ "Command requires an argument.",
 	/* 28 */ "Wrong character after backslash.",
-	/* 29 */ "Can't open file.",
-	/* 30 */ "Can't open temporary files.",
-	/* 31 */ "Error while writing.",
-	/* 32 */ "Document name has no extension.",
-	/* 33 */ "Can't find or create $HOME/.ne directory.",
-	/* 34 */ "Clip does not exist.",
-	/* 35 */ "Mark is out of document.",
-	/* 36 */ "Can't open macro.",
-	/* 37 */ "Maximum macro depth exceeded.",
-	/* 38 */ "This document is read-only.",
-	/* 39 */ "Can't open file (file is migrated).",
-	/* 40 */ "Can't open file (file is a directory).",
-	/* 41 */ "Can't open file (file is too large).",
-	/* 42 */ "Stopped.",
-	/* 43 */ "I/O error.",
-	/* 44 */ "The argument string is empty.",
-	/* 45 */ "External command error.",
-	/* 46 */ "Escape time out of range.",
-	/* 47 */ "Prefs stack is full.",
-	/* 48 */ "Prefs stack is empty.",
-	/* 49 */ "The argument is not a number.",
-	/* 50 */ "This character is not supported in this configuration.",
-	/* 51 */ "This string is not supported in this configuration.",
-	/* 52 */ "This buffer is not UTF-8 encoded.",
-	/* 53 */ "This clip cannot be pasted in this buffer (incompatible encoding).",
-	/* 54 */ "This command line cannot be executed in this buffer (incompatible encoding).",
-	/* 55 */ "This string cannot be searched for in this buffer (incompatible encoding).",
-	/* 56 */ "This replacement string cannot be used in this buffer (incompatible encoding).",
-	/* 57 */ "UTF-8 character classes in regular expressions are not supported.",
-	/* 58 */ "Character classes cannot be complemented when matching against UTF-8 text.",
-	/* 59 */ "The specified regex replacement group is not available in UTF-8 mode",
-	/* 60 */ "Syntax highlighting is not enabled",
-	/* 61 */ "There is no syntax for that extension",
-	/* 62 */ "Invalid Shift specified (use [<|>][#][s|t]; default is \">1t\").",
-	/* 63 */ "Insufficient white space for requested left shift.",
-	/* 64 */ "Document not saved.",
-	/* 65*/	"File is too large--syntax highlighting disabled (use SYNTAX to reactivate).",
-	/* 66*/	"Cannot save: disk full.",
-	/* 67*/	"Out of memory (insufficient disk space?). DANGER!"
+	/* 29 */ "File doesn't exist.",
+	/* 30 */ "Can't open file.",
+	/* 31 */ "Can't open temporary files.",
+	/* 32 */ "Error while writing.",
+	/* 33 */ "Document name has no extension.",
+	/* 34 */ "Can't find or create $HOME/.ne directory.",
+	/* 35 */ "Clip does not exist.",
+	/* 36 */ "Mark is out of document.",
+	/* 37 */ "Can't open macro.",
+	/* 38 */ "Maximum macro depth exceeded.",
+	/* 39 */ "This document is read-only.",
+	/* 40 */ "Can't open file (file is migrated).",
+	/* 41 */ "Can't open file (file is a directory).",
+	/* 42 */ "Can't open file (file is too large).",
+	/* 43 */ "Stopped.",
+	/* 44 */ "I/O error.",
+	/* 45 */ "The argument string is empty.",
+	/* 46 */ "External command error.",
+	/* 47 */ "Escape time out of range.",
+	/* 48 */ "Prefs stack is full.",
+	/* 49 */ "Prefs stack is empty.",
+	/* 50 */ "The argument is not a number.",
+	/* 51 */ "This character is not supported in this configuration.",
+	/* 52 */ "This string is not supported in this configuration.",
+	/* 53 */ "This buffer is not UTF-8 encoded.",
+	/* 54 */ "This clip cannot be pasted in this buffer (incompatible encoding).",
+	/* 55 */ "This command line cannot be executed in this buffer (incompatible encoding).",
+	/* 56 */ "This string cannot be searched for in this buffer (incompatible encoding).",
+	/* 57 */ "This replacement string cannot be used in this buffer (incompatible encoding).",
+	/* 58 */ "UTF-8 character classes in regular expressions are not supported.",
+	/* 59 */ "Character classes cannot be complemented when matching against UTF-8 text.",
+	/* 60 */ "The specified regex replacement group is not available in UTF-8 mode.",
+	/* 61 */ "Syntax highlighting is not enabled.",
+	/* 62 */ "There is no syntax for that extension.",
+	/* 63 */ "Invalid Shift specified (use [<|>][#][s|t]; default is \">1t\").",
+	/* 64 */ "Insufficient white space for requested left shift.",
+	/* 65 */ "Document not saved.",
+	/* 66 */ "File is too large--syntax highlighting disabled (use SYNTAX to reactivate).",
+	/* 67 */ "Cannot save: disk full.",
+	/* 68 */ "Out of memory (insufficient disk space?). DANGER!"
 };
 
 char *info_msg[INFO_COUNT] = {
 	"Saving...",
 	"Saved.",
 	"All modified documents saved.",
+	"Document is marked ReadOnly; save anyway?",
 	"SELECT cursor/Enter FILTER chars/BS MODE Ins/Del EDIT Tab FILENAME F1/Esc-Esc/Esc",
 	"Start of block marked. Move to the end of block and request actions.",
 	"Start of vertical block marked. Move to the end of block and request actions.",
 	"Starting macro recording...",
 	"Macro recording completed.",
+	"Macro record appending started.",
+	"Macro recording cancelled.",
 	"Some documents have not been saved; are you sure?",
 	"Press a key to see ne's corresponding key code:",
 	"This document is not saved; are you sure?",
@@ -115,6 +119,11 @@ char *info_msg[INFO_COUNT] = {
 	"SELECT cursor/Enter FILTER chars/BS MODE Ins/Del REORDER F2/F3 QUIT F1/Esc-Esc/Esc",
 	"File has been modified since buffer was loaded or saved; are you sure?",
 	"A file with that name already exists; are you sure?",
+	"No such file exists; blank out current document?",
 	"HELP cursor/Enter QUIT F1/Esc-Esc/Esc",
-	"BACK Enter QUIT F1/Esc-Esc/Esc"
+	"BACK Enter QUIT F1/Esc-Esc/Esc",
+	" (browse history with ^F)",
+	"Invalid argument for 'Record' while recording.",
+	"All Bookmarks cleared.",
+	"AtomicUndo level: 0"
 };
