@@ -1,7 +1,7 @@
 /* Error index enum, and extern declaration of the error vector.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -72,45 +72,46 @@ enum error {
 	/* 26 */ HAS_NO_ARGUMENT,
 	/* 27 */ REQUIRES_ARGUMENT,
 	/* 28 */ WRONG_CHAR_AFTER_BACKSLASH,
-	/* 29 */ CANT_OPEN_FILE,
-	/* 30 */ CANT_OPEN_TEMPORARY_FILE,
-	/* 31 */ ERROR_WHILE_WRITING,
-	/* 32 */ HAS_NO_EXTENSION,
-	/* 33 */ CANT_FIND_PREFS_DIR,
-	/* 34 */ CLIP_DOESNT_EXIST,
-	/* 35 */ MARK_OUT_OF_BUFFER,
-	/* 36 */ CANT_OPEN_MACRO,
-	/* 37 */ MAX_MACRO_DEPTH_EXCEEDED,
-	/* 38 */ DOCUMENT_IS_READ_ONLY,
-	/* 39 */ FILE_IS_MIGRATED,
-	/* 40 */ FILE_IS_DIRECTORY,
-	/* 41 */ FILE_IS_TOO_LARGE,
-	/* 42 */ STOPPED,
-	/* 43 */ IO_ERROR,
-	/* 44 */ STRING_IS_EMPTY,
-	/* 45 */ EXTERNAL_COMMAND_ERROR,
-	/* 46 */ ESCAPE_TIME_OUT_OF_RANGE,
-	/* 47 */ PREFS_STACK_FULL,
-	/* 48 */ PREFS_STACK_EMPTY,
-	/* 49 */ NOT_A_NUMBER,
-	/* 50 */ INVALID_CHARACTER,
-	/* 51 */ INVALID_STRING,
-	/* 52 */ BUFFER_IS_NOT_UTF8,
-	/* 53 */ INCOMPATIBLE_CLIP_ENCODING,
-	/* 54 */ INCOMPATIBLE_COMMAND_ENCODING,
-	/* 55 */ INCOMPATIBLE_SEARCH_STRING_ENCODING,
-	/* 56 */ INCOMPATIBLE_REPLACE_STRING_ENCODING,
-	/* 57 */ UTF8_REGEXP_CHARACTER_CLASS_NOT_SUPPORTED,
-	/* 58 */ UTF8_REGEXP_COMP_CHARACTER_CLASS_NOT_SUPPORTED,
-	/* 59 */ GROUP_NOT_AVAILABLE,
-	/* 60 */ SYNTAX_NOT_ENABLED,
-	/* 61 */ NO_SYNTAX_FOR_EXT,
-	/* 62 */ INVALID_SHIFT_SPECIFIED,
-	/* 63 */ INSUFFICIENT_WHITESPACE,
-	/* 64 */ DOCUMENT_NOT_SAVED,
-	/* 65 */ FILE_TOO_LARGE_SYNTAX_HIGHLIGHTING_DISABLED,
-	/* 66 */ CANNOT_SAVE_DISK_FULL,
-	/* 67 */ OUT_OF_MEMORY_DISK_FULL,
+	/* 29 */ FILE_DOES_NOT_EXIST,
+	/* 30 */ CANT_OPEN_FILE,
+	/* 31 */ CANT_OPEN_TEMPORARY_FILE,
+	/* 32 */ ERROR_WHILE_WRITING,
+	/* 33 */ HAS_NO_EXTENSION,
+	/* 34 */ CANT_FIND_PREFS_DIR,
+	/* 35 */ CLIP_DOESNT_EXIST,
+	/* 36 */ MARK_OUT_OF_BUFFER,
+	/* 37 */ CANT_OPEN_MACRO,
+	/* 38 */ MAX_MACRO_DEPTH_EXCEEDED,
+	/* 39 */ DOCUMENT_IS_READ_ONLY,
+	/* 40 */ FILE_IS_MIGRATED,
+	/* 41 */ FILE_IS_DIRECTORY,
+	/* 42 */ FILE_IS_TOO_LARGE,
+	/* 43 */ STOPPED,
+	/* 44 */ IO_ERROR,
+	/* 45 */ STRING_IS_EMPTY,
+	/* 46 */ EXTERNAL_COMMAND_ERROR,
+	/* 47 */ ESCAPE_TIME_OUT_OF_RANGE,
+	/* 48 */ PREFS_STACK_FULL,
+	/* 49 */ PREFS_STACK_EMPTY,
+	/* 50 */ NOT_A_NUMBER,
+	/* 51 */ INVALID_CHARACTER,
+	/* 52 */ INVALID_STRING,
+	/* 53 */ BUFFER_IS_NOT_UTF8,
+	/* 54 */ INCOMPATIBLE_CLIP_ENCODING,
+	/* 55 */ INCOMPATIBLE_COMMAND_ENCODING,
+	/* 56 */ INCOMPATIBLE_SEARCH_STRING_ENCODING,
+	/* 57 */ INCOMPATIBLE_REPLACE_STRING_ENCODING,
+	/* 58 */ UTF8_REGEXP_CHARACTER_CLASS_NOT_SUPPORTED,
+	/* 59 */ UTF8_REGEXP_COMP_CHARACTER_CLASS_NOT_SUPPORTED,
+	/* 60 */ GROUP_NOT_AVAILABLE,
+	/* 61 */ SYNTAX_NOT_ENABLED,
+	/* 62 */ NO_SYNTAX_FOR_EXT,
+	/* 63 */ INVALID_SHIFT_SPECIFIED,
+	/* 64 */ INSUFFICIENT_WHITESPACE,
+	/* 65 */ DOCUMENT_NOT_SAVED,
+	/* 66 */ FILE_TOO_LARGE_SYNTAX_HIGHLIGHTING_DISABLED,
+	/* 67 */ CANNOT_SAVE_DISK_FULL,
+	/* 68 */ OUT_OF_MEMORY_DISK_FULL,
 
 	ERROR_COUNT
 };
@@ -119,11 +120,14 @@ enum info {
 	SAVING,
 	SAVED,
 	MODIFIED_SAVED,
+	SAVE_READ_ONLY_DOCUMENT,
 	PRESSF1,
 	BLOCK_START_MARKED,
 	VERTICAL_BLOCK_START_MARKED,
 	STARTING_MACRO_RECORDING,
 	MACRO_RECORDING_COMPLETED,
+	MACRO_RECORD_APPENDING_STARTED,
+	MACRO_RECORDING_CANCELLED,
 	SOME_DOCUMENTS_ARE_NOT_SAVED,
 	PRESS_A_KEY,
 	THIS_DOCUMENT_NOT_SAVED,
@@ -135,8 +139,13 @@ enum info {
 	SELECT_DOC,
 	FILE_HAS_BEEN_MODIFIED,
 	FILE_ALREADY_EXISTS,
+	NO_SUCH_FILE_EXISTS,
 	HELP_KEYS,
 	HELP_COMMAND_KEYS,
+	LONG_INPUT_HELP,
+	INVALID_ARGUMENT_WHILE_RECORDING,
+	ALL_BOOKMARKS_CLEARED,
+	ATOMIC_UNDO_LEVEL_0,
 
 	INFO_COUNT
 };
