@@ -1,7 +1,7 @@
 /* Help strings.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -454,11 +454,11 @@ const char * const EXIT_HELP[12] = {
   "saved, so 'Exit' will report an error if you have any modified unnamed",
   "documents.  Like 'SaveAll', 'Exit' will not save a document if its",
   "corresponding file has been modified since the document was loaded or",
-  "last saved, in which case an error is reported and no documents are",
-  "closed.",
+  "last saved, or if its read only flag is set, in which cases an error is",
+  "reported and no documents are closed.",
 };
 
-const char * const FASTGUI_HELP[19] = {
+const char * const FASTGUI_HELP[20] = {
   "Syntax: FastGUI [0|1]",
   "Abbreviation: FG",
   "",
@@ -466,9 +466,10 @@ const char * const FASTGUI_HELP[19] = {
   "'ne' tries to print as little as possible while displaying menus and the",
   "status bar.  In particular, menu items are highlighted by the cursor",
   "only, the status bar is not highlighted (which allows printing it with",
-  "fewer characters) and the hexadecimal code is not displayed.  This",
-  "option is only (but very) useful if you are using 'ne' through a slow",
-  "connection.",
+  "fewer characters), the current position of the mark is not highlighted,",
+  "and the hexadecimal code for the character under the cursor is not",
+  "displayed.  This option is only (but very) useful if you are using 'ne'",
+  "through a slow connection.",
   "",
   "   If you invoke 'FastGUI' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
@@ -507,7 +508,7 @@ const char * const FINDREGEXP_HELP[12] = {
   "on the input line, the default being the last pattern used.",
 };
 
-const char * const FLAGS_HELP[35] = {
+const char * const FLAGS_HELP[36] = {
   "Syntax: Flags",
   "Abbreviation: FLAG",
   "",
@@ -524,9 +525,9 @@ const char * const FLAGS_HELP[35] = {
   "       p   AutoPrefs     AP   use automatic preferences based on file extension",
   "       v   VerboseMacros VM   record macros using use long command names",
   "       u   DoUndo        DU   record edits for later undoing",
-  "       r   ReadOnly      RO   changes are not allowed",
+  "       r   ReadOnly      RO   changes are not allowed/saves are prompted",
   "       t/T Tabs          TAB  TAB key inserts TABs instead of spaces",
-  "       T   ShiftTabs     ST   Shift may insert TABs (only if 't' is also set)",
+  "       T   ShiftTabs     SHT  Shift may insert TABs (only if 't' is also set)",
   "       d   DelTabs       DT   BS and DEL may remove tabs worth of space",
   "       B/! Binary        B    affects file loading/saving",
   "       M   Mark          M    mark set for line-oriented block operations",
@@ -534,9 +535,10 @@ const char * const FLAGS_HELP[35] = {
   "       R   Record        REC  actions are being recorded in a macro",
   "       P   PreserveCR    PCR  affects how <CR> chars are loaded from files",
   "       C   CRLF          CRLF use CR/LF as line terminator",
-  "       *   Modified      MOD  document has been modified since last saved",
   "       @   UTF8IO        U8IO I/O (keyboard and terminal) are UTF-8 encoded",
   "     A/8/U UTF8          U8   the document encoding (ASCII, 8-bit or UTF-8)",
+  "       */_ Modified      MOD  document has been modified since last saved",
+  "       _   (none)             file's modtime changed since doc was loaded/saved",
   "",
   "   The 'RequestOrder' and 'AutoMatchBracket' flags' states are not",
   "indicated on the status bar.  See `RequestOrder' and ",
@@ -579,7 +581,7 @@ const char * const GOTOBOOKMARK_HELP[20] = {
   "moves the cursor to the designated bookmark if that bookmark is set; see",
   "`SetBookmark'.  Each document has 10 available bookmarks",
   "designated '0' to '9', plus the automatic bookmark designated by '-'.",
-  "If no option is given, '0' is assumed.  The optons '+1' and '-1'",
+  "If no option is given, '0' is assumed.  The options '+1' and '-1'",
   "indicate respectively the next and previous set bookmarks, so that",
   "repeated 'GotoBookmark +1' commands will cycle through all currently set",
   "bookmarks.  When successful, the '-' automatic bookmark is set to the",
@@ -624,7 +626,7 @@ const char * const GOTOMARK_HELP[9] = {
   "",
   "   'GotoMark' is mainly useful if you forgot where you started marking.",
   "If you want to record positions in a file and jump to them later, you",
-  "may want to use a bookmarks.  See `SetBookmark'.",
+  "may want to use bookmarks instead.  See `SetBookmark'.",
 };
 
 const char * const HELP_HELP[14] = {
@@ -818,10 +820,10 @@ const char * const MARK_HELP[12] = {
   "",
   "sets the mark at the current position or cancels the previous mark.  The",
   "mark and cursor together define the range of text over which clips",
-  "('Cut', 'Copy', 'Erase') and left and right shifts operate .",
+  "('Cut', 'Copy', 'Erase') and left and right shifts operate.",
   "",
   "   If you invoke 'Mark' with no arguments, it will set the mark.  If you",
-  "specify 0 or 1, the mark will be canceled or set to the current",
+  "specify 0 or 1, the mark will be cancelled or set to the current",
   "position, respectively.  A capital 'M' appears on the status bar, if the",
   "mark is active.",
 };
@@ -833,7 +835,7 @@ const char * const MARKVERT_HELP[21] = {
   "is the same as 'Mark', but the region manipulated by the cut/paste",
   "commands is the rectangle having as vertices the cursor and the mark.",
   "If you invoke 'MarkVert' with no arguments, it will set the mark.  If",
-  "you specify 0 or 1, the mark will be canceled or set to the current",
+  "you specify 0 or 1, the mark will be cancelled or set to the current",
   "position, respectively.  Moreover, a capital 'V', rather than a capital",
   "'M', will appear on the status bar.",
   "",
@@ -860,21 +862,23 @@ const char * const MATCHBRACKET_HELP[9] = {
   "`AutoMatchBracket'.",
 };
 
-const char * const MODIFIED_HELP[15] = {
+const char * const MODIFIED_HELP[17] = {
   "Syntax: Modified [0|1]",
   "Abbreviation: MOD",
   "",
   "sets the modified flag.  This flag is set automatically whenever a",
   "document is modified, and is used to determine which documents need to",
-  "be saved when 'ne' exits.  Normally you would not alter this flag, but",
-  "when a document is inadvertently modified and you don't want the changes",
-  "saved, 'Modified' provides a way to make 'ne' consider the document",
-  "unchanged.",
+  "be saved by 'SaveAll' or when 'ne' exits.  Normally you would not alter",
+  "this flag, but when a document is inadvertently modified and you don't",
+  "want the changes saved, 'Modified' provides a way to make 'ne' consider",
+  "the document unchanged.",
   "",
   "   If you invoke 'Modified' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
-  "respectively.  An asterisk ('*') will appear on the status bar if the",
-  "flag is true.",
+  "respectively.  An asterisk ('*') will appear on the status bar when this",
+  "flag is set.  If your terminal supports underlining, it will be",
+  "underlined if the corresponding file's modification time has changed",
+  "since the document was loaded or saved (perhaps by another user).",
 };
 
 const char * const MOVEBOS_HELP[6] = {
@@ -899,11 +903,14 @@ const char * const MOVEEOL_HELP[5] = {
   "moves the cursor to the end of the current line ('EOL' = end of line).",
 };
 
-const char * const MOVEEOW_HELP[8] = {
-  "Syntax: MoveEOW",
+const char * const MOVEEOW_HELP[11] = {
+  "Syntax: MoveEOW [<|>]",
   "Abbreviation: EOW",
   "",
-  "moves the cursor one character past the end of the current word.",
+  "moves the cursor to the end of the current word, that end being the",
+  "right end unless you include the optional parameter '<'.  If the cursor",
+  "is not currently in a word, or if it's already at the indicated end, it",
+  "retains its current location.",
   "",
   "   'MoveEOW' is extremely useful in macros, because it allows you to",
   "copy precisely the word the cursor is on.",
@@ -1008,12 +1015,14 @@ const char * const NEXTPAGE_HELP[8] = {
   "is assumed to be one.",
 };
 
-const char * const NEXTWORD_HELP[6] = {
-  "Syntax: NextWord [N]",
+const char * const NEXTWORD_HELP[8] = {
+  "Syntax: NextWord [N][<|>]",
   "Abbreviation: NW",
   "",
-  "moves the cursor to the next word N times.  If the optional N argument",
-  "is not specified, it is assumed to be one.",
+  "moves the cursor to the right to the Nth word beginning or ending.  If",
+  "the optional N argument is not specified, one is used.  The optional '<'",
+  "or '>' determines whether to move to the word beginning or ending,",
+  "respectively.  If unspecified, the left end ('<') is used.",
 };
 
 const char * const NOFILEREQ_HELP[10] = {
@@ -1035,12 +1044,15 @@ const char * const NOP_HELP[5] = {
   "does nothing.  Mainly useful for inhibiting standard key bindings.",
 };
 
-const char * const OPEN_HELP[18] = {
+const char * const OPEN_HELP[21] = {
   "Syntax: Open [FILENAME]",
   "Abbreviation: O",
   "",
-  "loads the file specified by the FILENAME string into the current",
-  "document.",
+  "replaces the contents of the current document with that of the file",
+  "specified by the FILENAME string.  (To load FILENAME's content into a",
+  "new document without changing the current document, see ",
+  "`OpenNew'.)  The current document's macro, search, and replace strings",
+  "are preserved.",
   "",
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file.  (You can",
@@ -1075,8 +1087,8 @@ const char * const OPENMACRO_HELP[14] = {
   "Syntax: OpenMacro [FILENAME]",
   "Abbreviation: OM",
   "",
-  "loads the given file name as the current macro just as if you 'Record'ed",
-  "it; see `Record'.",
+  "loads the given file name as the current document's macro just as if you",
+  "'Record'ed it; see `Record'.",
   "",
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file.  (You can",
@@ -1087,12 +1099,17 @@ const char * const OPENMACRO_HELP[14] = {
   "the command line.",
 };
 
-const char * const OPENNEW_HELP[6] = {
+const char * const OPENNEW_HELP[11] = {
   "Syntax: OpenNew [FILENAME]",
   "Abbreviation: ON",
   "",
-  "is the same as 'Open', but loads the file specified by the FILENAME",
-  "string into a new document.  See `Open'.",
+  "creates a new document and loads into it the contents of the file",
+  "specified by the optional FILENAME string.  This new document will",
+  "inherit the macro, search, and replace strings from the current",
+  "document.",
+  "",
+  "   If FILENAME is unspecified, the file requester behaves the same as",
+  "for the 'Open' command; see `Open'.",
 };
 
 const char * const PAGEDOWN_HELP[6] = {
@@ -1111,7 +1128,7 @@ const char * const PAGEUP_HELP[6] = {
   "assumed to be one.",
 };
 
-const char * const PARAGRAPH_HELP[27] = {
+const char * const PARAGRAPH_HELP[24] = {
   "Syntax: Paragraph [N]",
   "Abbreviation: PA",
   "",
@@ -1124,10 +1141,7 @@ const char * const PARAGRAPH_HELP[27] = {
   "(regardless of its leading white space) and all subsequent non-blank",
   "lines that have identical (to each other's--not to the first line's)",
   "leading white space.  Therefore your paragraphs can have various first",
-  "line indentations and left margins.  'Paragraph' also takes into account",
-  "characters commonly used at the left edge of block comments ('/', '*',",
-  "'#', spaces, and tabs) or quoted email ('>'), and attempts to preserve",
-  "those on the left edge when possible.",
+  "line indentations and left margins.",
   "",
   "   After the 'Paragraph' command completes, your cursor will be",
   "positioned on the first non-blank character after the last reformatted",
@@ -1158,12 +1172,13 @@ const char * const PASTEVERT_HELP[7] = {
   "lines at the horizontal cursor position.",
 };
 
-const char * const PLAY_HELP[15] = {
+const char * const PLAY_HELP[16] = {
   "Syntax: Play [TIMES]",
   "Abbreviation: PL",
   "",
-  "plays the current macro for the given number of times.  If the optional",
-  "argument TIMES is not specified, you can enter it on the input line.",
+  "plays the current document's macro the given number of times.  If the",
+  "optional argument TIMES is not specified, you can enter it on the input",
+  "line.",
   "",
   "   A (possibly iterated) macro execution terminates as soon as its",
   "stream of instructions is exhausted, or one of its commands returns an",
@@ -1237,13 +1252,13 @@ const char * const PREVPAGE_HELP[8] = {
 };
 
 const char * const PREVWORD_HELP[8] = {
-  "Syntax: PrevWord [N]",
+  "Syntax: PrevWord [N][<|>]",
   "Abbreviation: PW",
   "",
-  "moves the cursor to the first character of the previous word N times.",
-  "If the optional N argument is not specified, it is assumed to be one (in",
-  "which case, if the cursor is in the middle of a word the effect is just",
-  "to move it to the start of that word).",
+  "moves the cursor to the left to the Nth word beginning or ending.  If",
+  "the optional N argument is not specified, one is used.  The optional '<'",
+  "or '>' determines whether to move to the word beginning or ending,",
+  "respectively.  If unspecified, the left end ('<') is used.",
 };
 
 const char * const PUSHPREFS_HELP[24] = {
@@ -1280,14 +1295,16 @@ const char * const QUIT_HELP[6] = {
   "to confirm the action.",
 };
 
-const char * const READONLY_HELP[13] = {
+const char * const READONLY_HELP[15] = {
   "Syntax: ReadOnly [0|1]",
   "Abbreviation: RO",
   "",
   "sets the read only flag.  When this flag is true, no editing can be",
   "performed on the document (any such attempt produces an error message).",
-  "This flag is automatically set whenever you open a file that you cannot",
-  "write to.  See `Open'.",
+  "Saving read only documents is inhibited as well; you must affirmatively",
+  "answer a prompt to save a document with the read only flag set.  This",
+  "flag is automatically set whenever you open a file that you cannot write",
+  "to.  See `Open'.",
   "",
   "   If you invoke 'ReadOnly' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
@@ -1295,27 +1312,28 @@ const char * const READONLY_HELP[13] = {
   "flag is true.",
 };
 
-const char * const RECORD_HELP[21] = {
+const char * const RECORD_HELP[22] = {
   "Syntax: Record [0|1]",
   "Abbreviation: REC",
   "",
-  "sets the recording state flag.  When this flag becomes true, 'ne' starts",
-  "recording your actions in a new macro.  When it becomes false, the macro",
-  "recording is stopped, and the macro can be played or saved via ",
-  "`Play', or `SaveMacro'.",
+  "starts, stops, cancels, or resumes macro recording.  With no arguments,",
+  "'Record' starts recording your commands as a new macro unless recording",
+  "is already underway.  In that case, macro recording is stopped, and the",
+  "newly recorded macro replaces the current document's unnamed macro.  The",
+  "new macro can be played or saved via `Play', or `SaveMacro'.",
+  "The default key binding for '<Control>-T' is 'Record' with no arguments,",
+  "and is by far the most common way to use the 'Record' command.",
   "",
-  "   If you invoke 'Record' with no arguments, it will toggle the flag.",
-  "If you specify 0 or 1, the flag will be set to false or true,",
-  "respectively.  An upper case 'R' will appear on the status bar if the",
-  "flag is true.",
+  "   If you've started recording a macro and wish to cancel rather than",
+  "wipe out your existing macro, you can cancel the recording by using",
+  "'Record 0'.  An error will be displayed if you aren't recording already,",
+  "but it's harmless.",
   "",
-  "   The reason for providing a flag instead of an explicit start/stop",
-  "recording command pair is that this way it is possible to bind both",
-  "starting and stopping macro recording to a single key while still being",
-  "able to specify \"absolute\" menu items (by using 'Record 0' and 'Record",
-  "1').  For instance, the default key binding for '<Control>-T' is simply",
-  "'Record', which means that this shortcut can be used both for initiating",
-  "and for terminating a macro recording.",
+  "   Sometimes you've got a macro either recorded or loaded from a file",
+  "(see `OpenMacro'), but you'd like to record additional commands",
+  "onto the end of it.  'Record 1' will do that.  It will start recording",
+  "onto the end of a copy of your current document's macro.  An error will",
+  "be displayed if you are already recording, but it's otherwise harmless.",
 };
 
 const char * const REDO_HELP[8] = {
@@ -1433,7 +1451,7 @@ const char * const REPLACEALL_HELP[14] = {
   "   If the optional argument STRING is not specified, you can enter it on",
   "the input line, the default being the last string used.",
   "",
-  "   Note that a slingle 'Undo' will restore _all_ the occurrences of the",
+  "   Note that a single 'Undo' will restore _all_ the occurrences of the",
   "search pattern replaced by 'ReplaceAll'.  See `Undo'.",
 };
 
@@ -1480,7 +1498,7 @@ const char * const RIGHTMARGIN_HELP[12] = {
   "the current screen width as right margin.",
 };
 
-const char * const SAVE_HELP[17] = {
+const char * const SAVE_HELP[18] = {
   "Syntax: Save",
   "Abbreviation: S",
   "",
@@ -1496,22 +1514,24 @@ const char * const SAVE_HELP[17] = {
   "",
   "   If the file has been modified since the current document was loaded",
   "or last saved (perhaps by another user), 'ne' will warn you before",
-  "overwriting the updated file.",
+  "overwriting the updated file.  If the current document's read only flag",
+  "is set, 'ne' will prompt you before attempting to save it.",
 };
 
-const char * const SAVEALL_HELP[10] = {
+const char * const SAVEALL_HELP[11] = {
   "Syntax: SaveAll",
   "Abbreviation: SL",
   "",
   "saves all modified documents.  If any modified documents cannot be",
   "saved, the action is suspended and an error message is issued.  Note",
   "that only named documents can be saved, so 'SaveAll' will report an",
-  "error if you have any modified unnamed documents, or if any of the",
-  "modified documents' corresponding files have been updated since they",
-  "were loaded or last saved.",
+  "error if you have any modified unnamed documents.  Other reasons",
+  "'SaveAll' may fail include: if any of the modified documents'",
+  "corresponding files have been updated since they were loaded or last",
+  "saved, or if any modified documents' read only flags are set.",
 };
 
-const char * const SAVEAS_HELP[18] = {
+const char * const SAVEAS_HELP[19] = {
   "Syntax: SaveAs [FILENAME]",
   "Abbreviation: SA",
   "",
@@ -1528,7 +1548,8 @@ const char * const SAVEAS_HELP[18] = {
   "",
   "   If the file has been modified since the current document was loaded",
   "or last saved (perhaps by another user), 'ne' will warn you before",
-  "overwriting the updated file.",
+  "overwriting the updated file.  If the current document's read only flag",
+  "is set, 'ne' will prompt you before attempting to save it.",
 };
 
 const char * const SAVEAUTOPREFS_HELP[7] = {
@@ -1567,7 +1588,7 @@ const char * const SAVEMACRO_HELP[20] = {
   "Syntax: SaveMacro [FILENAME]",
   "Abbreviation: SM",
   "",
-  "saves the current macro in a file with the given name.",
+  "saves the current document's macro in a file with the given name.",
   "",
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file.  (You can",
@@ -1614,7 +1635,7 @@ const char * const SEARCHBACK_HELP[14] = {
   "'Replace' command.  See `Replace'.",
 };
 
-const char * const SELECTDOC_HELP[24] = {
+const char * const SELECTDOC_HELP[25] = {
   "Syntax: SelectDoc",
   "Abbreviation: SD",
   "",
@@ -1626,10 +1647,11 @@ const char * const SELECTDOC_HELP[24] = {
   "and hitting <Return>..",
   "",
   "   While the list of documents is displayed, you can close unmodified",
-  "documents with your 'CloseDoc' key ('<Control>-Q'), and you can alter",
-  "their relative order using your 'NextDoc' and 'PrevDoc' keys (<f2> and",
-  "<f3>), which have the effect of swapping your currently selected",
-  "document with the next or previous document in the list, respectively.",
+  "documents with your 'CloseDoc' key ('<Control>-Q'), save named documents",
+  "with your 'Save' key ('<Control>-S'), and you can alter their relative",
+  "order using your 'NextDoc' and 'PrevDoc' keys (<f2> and <f3>), which",
+  "have the effect of swapping your currently selected document with the",
+  "next or previous document in the list, respectively.",
   "",
   "   If you escape from the requester, the requester goes away, you are",
   "returned to your original current document (unless you closed it!), and",
@@ -1660,22 +1682,22 @@ const char * const SETBOOKMARK_HELP[17] = {
 };
 
 const char * const SHIFT_HELP[21] = {
-  "Syntax: Shift [<|>][n][t|s]]",
+  "Syntax: Shift [<|>][n][t|s]",
   "Abbreviation: SH",
   "",
   "shifts the text on lines between the mark and the cursor either right",
-  "('>', the default) or left ('<') by adding or removing white space on",
-  "each line.  The adjustment size, specified as an unsigned integer 'n',",
-  "is in units of the current tab size ('t') or spaces ('s').  The default",
-  "is 1.  Adjustments start at the left edge of a vertical mark, or column",
-  "1 otherwise.  If the mark is not currently set, only the current line is",
-  "affected.",
+  "('>') or left ('<') by adding or removing white space on each line.  The",
+  "adjustment size, specified as an unsigned integer 'n', is in units of",
+  "the current tab size ('t') or spaces ('s').  The defaults are '>', '1',",
+  "and 't'.  Adjustments start at the left edge of a vertical mark, or",
+  "column 1 otherwise.  If the mark is not currently set, only the current",
+  "line is affected.",
   "",
-  "   'Shift' will insert tab characters only if the document's 'Tabs' flag",
-  "and the 'ShiftTabs' flag are both set--in which case an upper case 'T'",
-  "will appear in the status bar.  If either of the 'Tabs' or 'ShiftTabs'",
-  "flags is unset (i.e there is no upper case 'T' in the status bar)",
-  "'Shift' will only insert spaces.",
+  "   'Shift' will insert tab characters only if 's' is not used, and both",
+  "of the document's 'Tabs' and 'ShiftTabs' flags are set--in which case an",
+  "upper case 'T' will appear in the status bar.  If either of the 'Tabs'",
+  "or 'ShiftTabs' flags is unset (i.e there is no upper case 'T' in the",
+  "status bar) 'Shift' will only insert spaces.",
   "",
   "   In the case of left shifts, if any indicated line has insufficient",
   "leading white space for the requested adjustment to be made, then",
@@ -2054,16 +2076,13 @@ const char * const VISUALBELL_HELP[10] = {
   "respectively.",
 };
 
-const char * const WORDWRAP_HELP[15] = {
+const char * const WORDWRAP_HELP[12] = {
   "Syntax: WordWrap [0|1]",
   "Abbreviation: WW",
   "",
   "sets the word wrap flag.  When this flag is true, 'ne' will",
   "automatically break lines of text when you attempt to insert characters",
-  "beyond the right margin.  See `RightMargin'.  'ne' will attempt to",
-  "preserve certain invariant characters at the left edge of paragraphs",
-  "typically used to indicate comments in source code or quoted passages in",
-  "email text.  See `Paragraph'.",
+  "beyond the right margin.  See `RightMargin'.",
   "",
   "   If you invoke 'WordWrap' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
