@@ -1,4 +1,4 @@
-#!/usr/bin/perl -w
+#!/usr/bin/env -S perl -w
 
 use strict;
 $| = 1;
@@ -13,7 +13,7 @@ $| = 1;
 
 my $copyright =
   qq[	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -341,8 +341,8 @@ $copyright  */
    is updated. */
 
 /* Here we have the name of all commands. They are extern'd in names.h. Note
-that whenever you add or remove a command, you should immediately change also
-the command_names array at the end of this file. */
+   that whenever you add or remove a command, you should immediately change also
+   the command_names array at the end of this file. */
 
 
 _EOT_
@@ -378,9 +378,12 @@ _EOT_
 sub tail_names_c
   {
     print NAMES_C <<"_EOT_";
-/* These are extras that are very useful in the default menus and key bindings. */
+/* These are extras that are very useful in the default menus and key bindings.
+   See info2src.pl; changes here will be lost the next time ne.texinfo changes. */
 
 const char PLAYONCE_ABBREV[] = "PL 1";
+const char RECORD_CANCEL_ABBREV[] = "REC 0";
+const char RECORD_APPEND_ABBREV[] = "REC 1";
 const char MIDDLEVIEW_ABBREV[] = "AV M";
 const char SHIFTLEFT_ABBREV[] = "SH <";
 #ifdef ALTPAGING
@@ -409,6 +412,10 @@ sub head_names_h
 
 $copyright  */
 
+/* This file is generated by info2src.pl from data found in ne.texinfo.
+   Changes made directly to this file will we lost when the documentation
+   is updated. */
+
 
 _EOT_
   }
@@ -429,8 +436,12 @@ sub tail_names_h
   {
     print NAMES_H <<"_EOT_";
 
-/* These are extras that are very useful in the default menus and key bindings. */
+/* These are extras that are very useful in the default menus and key bindings.
+   See info2src.pl; changes here will be lost the next time ne.texinfo changes. */
+
 extern const char PLAYONCE_ABBREV[];
+extern const char RECORD_CANCEL_ABBREV[];
+extern const char RECORD_APPEND_ABBREV[];
 extern const char MIDDLEVIEW_ABBREV[];
 extern const char SHIFTLEFT_ABBREV[];
 #ifdef ALTPAGING
