@@ -1,7 +1,7 @@
 /* Input class, key bindings and upper casing vector definitions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -25,6 +25,10 @@ const char *input_class_names[INPUT_CLASS_COUNT] = {
 	"ALPHA", "COMMAND", "RETURN", "TAB", "IGNORE", "INVALID"
 }; 
 
+const char *config_source_names[CONFIG_SOURCE_COUNT] = {
+	"Built In", "Global Prefs", "User Prefs", "Current Directory"
+}; 
+
 /* This vector contains all key bindings. Each entry points to a command line
    to be executed when the corresponding keystroke is input. The index
    correspond to the ASCII code, and to the codes defined in keycodes.h for the
@@ -37,6 +41,9 @@ const char *input_class_names[INPUT_CLASS_COUNT] = {
   #define PICK(A,B) B
 #endif
 
+config_source key_binding_source[NUM_KEYS];
+
+
 const char *key_binding[NUM_KEYS] = {
 
 	/* Control-letter bindings (Ctrl-X)*  0..31  0..1f */
@@ -140,10 +147,10 @@ const char *key_binding[NUM_KEYS] = {
 	LINEDOWN_ABBREV, 
 	MOVELEFT_ABBREV, 
 	MOVERIGHT_ABBREV, 
-	PICK(MOVEINCUP_ABBREV,MOVESOL_ABBREV), 
-	PICK(MOVEINCDOWN_ABBREV,MOVEEOL_ABBREV), 
-	PICK(NEXTPAGE_ABBREV,PAGEDOWN_ABBREV),
-	PICK(PREVPAGE_ABBREV,PAGEUP_ABBREV), 
+	PICK(MOVEINCUP_ABBREV, MOVESOL_ABBREV), 
+	PICK(MOVEINCDOWN_ABBREV, MOVEEOL_ABBREV), 
+	PICK(NEXTPAGE_ABBREV, PAGEDOWN_ABBREV),
+	PICK(PREVPAGE_ABBREV, PAGEUP_ABBREV), 
 	LINEDOWN_ABBREV, 
 	LINEUP_ABBREV, 
 	NULL, 
@@ -210,8 +217,8 @@ const char *key_binding[NUM_KEYS] = {
 
    /* 336..351  150..15F */
    /* F16 */ NULL,
-   /* F17 */ PICK(NULL,SHIFTLEFT_ABBREV), /* Shift-F5 */
-   /* F18 */ PICK(NULL,SHIFT_ABBREV),     /* Shift-F6 */
+   /* F17 */ PICK(NULL, SHIFTLEFT_ABBREV), /* Shift-F5 */
+   /* F18 */ PICK(NULL, SHIFT_ABBREV),     /* Shift-F6 */
    /* F19 */ NULL,
    /* F20 */ NULL,
    /* F21 */ NULL,
@@ -222,8 +229,8 @@ const char *key_binding[NUM_KEYS] = {
    /* F26 */ NULL,
    /* F27 */ NULL,
    /* F28 */ NULL,
-   /* F29 */ PICK(NULL,SHIFTSPACELEFT_ABBREV),  /* Ctrl-F5 */
-   /* F30 */ PICK(NULL,SHIFTSPACERIGHT_ABBREV), /* Ctrl-F6 */
+   /* F29 */ PICK(NULL, SHIFTSPACELEFT_ABBREV),  /* Ctrl-F5 */
+   /* F30 */ PICK(NULL, SHIFTSPACERIGHT_ABBREV), /* Ctrl-F6 */
    /* F31 */ DELETEPREVWORD_ABBREV,             /* Ctrl-F7 */
    
 	/* 352..367  160..16F */
@@ -354,7 +361,7 @@ const char *key_binding[NUM_KEYS] = {
 	/* ESC \ */ NULL, 
 	/* ESC ] */ NULL, 
 	/* ESC ^ */ NULL,
-	/* ESC _ */ NULL 
+	/* ESC _ */ PICK(NULL, AUTOINDENT_ABBREV)
 
 };
 
@@ -400,7 +407,6 @@ const input_class char_class[256] = {
 	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA
 };
 
-
 const char meta_prefixed[128][3] = {
 	{ '\x1b', 0 },
 	{ '\x1b', 1 },
@@ -534,7 +540,7 @@ const char meta_prefixed[128][3] = {
 
 const char *key_stroke[NUM_KEYS] = {
 
-	/* Control-letter bindings (Ctrl-X)*  0..31  0..1f */
+	/* Control-letter bindings (Ctrl-X)*  [0..32)  [0..20) */
 
 	"^@",
 	"^A",
@@ -572,7 +578,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"^^", 
 	"^_", 
 
-	/* These keys map to strings, not commands. 31..126  20..7e */
+	/* These keys map to strings, not commands. [32..127)  [20..7f) */
 	" ", "!", "\"","#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", "/",   /* space ! " # $ % & ' ( ) * + , - . /  */
 	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?",   /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
 
@@ -582,7 +588,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~",        /* p q r s t u v w x y z { | } ~    */
 	"Del",                         /*  127  7f */
 
-	/* Control-meta-letter bindings (Ctrl-Alt-X)  128..159 80..9f */
+	/* Control-meta-letter bindings (Ctrl-Alt-X)  [128..160) [80..A0) */
 
 	"^[@",
 	"^[A",
@@ -618,18 +624,18 @@ const char *key_stroke[NUM_KEYS] = {
 	"^[^",
 	"^[_",
 
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 160..175  A0..AF */
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 176..191  B0..BF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [160..176)  [A0..B0) */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [176..192)  [B0..C0) */
 
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 192..207  C0..CF */
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 207..223  D0..DF */
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 224..239  E0..EF */
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 240..255  F0..FF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [192..208)  [C0..D0) */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [208..224)  [D0..E0) */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [224..240)  [E0..F0) */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [240..256)  [F0..100) */
 
 
 	/* The following bindings are for the terminfo extended codes (see keycodes.h). */
 
-	/* Cursor movement keys 256..271  100..10F */
+	/* Cursor movement keys [256..272)  [100..110) */
 	"", 
 	"Up",
 	"Down",
@@ -647,7 +653,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"", 
 	"",
 
-	/* Editing keys   272..287   110..11F */
+	/* Editing keys   [272..288)   [110..120) */
 	"Del",
 	"", 
 	"BackSp",
@@ -665,7 +671,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"", 
 	"",
 
-	/* Keypad keys and fake (simulated) menu key  288..303  120..12F */
+	/* Keypad keys and fake (simulated) menu key  [288..304)  [120..130) */
 	"KPSoF",
 	"KPPrPg",
 	"S/EoL",
@@ -683,9 +689,9 @@ const char *key_stroke[NUM_KEYS] = {
 	"", 
 	"",
 
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 304..319  130..13F */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [304..320)  [130..140) */
 
-	/* Function keys   320..335  140..14F  */
+	/* Function keys   [320..336)  [140..150)  */
 	"F0",
 	"F1",
 	"F2",
@@ -703,11 +709,11 @@ const char *key_stroke[NUM_KEYS] = {
 	"F14",
 	"F15",
 
-	"F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23", "F24", "F25", "F26", "F27", "F28", "F29", "F30", "F31",  /* 336..351  150..15F */
-	"F32", "F33", "F34", "F35", "F36", "F37", "F38", "F39", "F40", "F41", "F42", "F43", "F44", "F45", "F46", "F47",  /* 352..367  160..16F */
-	"F48", "F49", "F50", "F51", "F52", "F53", "F54", "F55", "F56", "F57", "F58", "F59", "F60", "F61", "F62", "F63",  /* 368..383  170..17F */
+	"F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23", "F24", "F25", "F26", "F27", "F28", "F29", "F30", "F31",  /* [336..352)  [150..160) */
+	"F32", "F33", "F34", "F35", "F36", "F37", "F38", "F39", "F40", "F41", "F42", "F43", "F44", "F45", "F46", "F47",  /* [352..368)  [160..170) */
+	"F48", "F49", "F50", "F51", "F52", "F53", "F54", "F55", "F56", "F57", "F58", "F59", "F60", "F61", "F62", "F63",  /* [368..384)  [170..180) */
 
-	/* Prefix-simulated META (ESC Ctrl-a)  384..416  180..1A0 */
+	/* Prefix-simulated META (ESC Ctrl-a)  [384..416)  [180..1A0) */
 
 	"ESC-^@",
 	"ESC-^A",
@@ -743,10 +749,42 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-^^",
 	"ESC-^_",
 
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 416..431  1A0..1AF */
-	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 432..447  1B0..1BF */
+	/* Prefix-simulated META (ESC Ctrl-a)  [416..447) [1A0..1C0) */
 
-	/* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
+	"ESC- ",
+	"ESC-!",
+	"ESC-\"",
+	"ESC-#",
+	"ESC-$",
+	"ESC-%",
+	"ESC-&",
+	"ESC-'",
+	"ESC-(",
+	"ESC-)",
+	"ESC-*",
+	"ESC-+",
+	"ESC-,",
+	"ESC--",
+	"ESC-.",
+	"ESC-/",
+	"ESC-0",
+	"ESC-1",
+	"ESC-2",
+	"ESC-3",
+	"ESC-4",
+	"ESC-5",
+	"ESC-6",
+	"ESC-7",
+	"ESC-8",
+	"ESC-9",
+	"ESC-:",
+	"ESC-;",
+	"ESC-<",
+	"ESC-=",
+	"ESC->",
+	"ESC-?",
+
+	/* Prefix-simulated META (ESC X)  [448..480)   [1C0..1E0) */
 	"ESC-@",
 	"ESC-A",
 	"ESC-B",
@@ -782,7 +820,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-_",
 
 
-	/* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
+	/* Prefix-simulated META (ESC X)  [480..512)  [1E0..200) */
 	"ESC-@",
 	"ESC-a",
 	"ESC-b",
@@ -811,10 +849,10 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-x",
 	"ESC-y",
 	"ESC-z",
-	"ESC-[",
-	"ESC-\\",
-	"ESC-]",
-	"ESC-^",
-	"ESC-_",
+	"ESC-{",
+	"ESC-|",
+	"ESC-}",
+	"ESC-~",
+	"ESC-DEL",
 
 };
