@@ -1,7 +1,7 @@
 /* Menu handling function. Includes also key and menu configuration parsing.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -48,17 +48,11 @@
 
 #define MENU_CONF_NAME ".menus"
 
-/* The name of the key bindings file. */
-
-#define KEY_BINDINGS_NAME ".keys"
-
 
 /* The keywords used in the configuration files. */
 
 #define MENU_KEYWORD "MENU"
 #define ITEM_KEYWORD "ITEM"
-#define KEY_KEYWORD "KEY"
-#define SEQ_KEYWORD "SEQ"
 
 
 /* This structure defines a menu item. command_line points to
@@ -154,6 +148,8 @@ static const menu_item search_item[] =
 static const menu_item macros_item[] =
 	{
 		{ "Start/Stop Rec ^T", RECORD_ABBREV },
+		{ "Record Cancel    ", RECORD_CANCEL_ABBREV },
+		{ "Record Append    ", RECORD_APPEND_ABBREV },
 		{ "Play Once   f9/[M", PLAYONCE_ABBREV },
 		{ "Play Many...     ", PLAY_ABBREV },
 		{ "Play Macro...    ", MACRO_ABBREV },
@@ -221,7 +217,7 @@ static const menu_item prefs_item[] =
 		{ "Fast GUI        ", FASTGUI_ABBREV },
 		{ "Word Wrap     [W", WORDWRAP_ABBREV },
 		{ "Right Margin    ", RIGHTMARGIN_ABBREV },
-		{ "Auto Indent     ", AUTOINDENT_ABBREV },
+PICK(   "Auto Indent     ", AUTOINDENT_ABBREV  , "Auto Indent [Del", AUTOINDENT_ABBREV)
 		{ "Request Order   ", REQUESTORDER_ABBREV },
 		{ "Preserve CR     ", PRESERVECR_ABBREV },
 		{ "Save CR/LF    [Z", CRLF_ABBREV },
@@ -313,11 +309,8 @@ static menu *menus = def_menus;
 
 
 #ifdef NE_TEST
-int dump_config(void) {
-	int menu, item, key;
-	FILE *f;
-
-	if (!(f = fopen("ne_test_dump_config", "w")) ) return ERROR;
+void dump_menu_config(FILE *f) {
+	int menu, item, keynum;
 
 	for (menu = 0; menu < menu_num; menu++) {
 		fprintf(f, "%s \"%s\"\n", MENU_KEYWORD, menus[menu].text );
@@ -328,13 +321,6 @@ int dump_config(void) {
 		}
 		fprintf(f, "\n");
 	}
-
-	for (key = 0; key < NUM_KEYS; key++) {
-		if (key_binding[key] && key_binding[key][0])
-			fprintf(f, "%s\t%4x\t%s\n", KEY_KEYWORD, key, key_binding[key] );
-	}
-	fclose(f);
-	return OK;
 }
 #endif
 
@@ -553,6 +539,7 @@ the flags are updated the cursor does not need to be moved after
 printing the numbers (an operation which usually needs the output
 of several characters). */
 
+static int mod_flag_index;
 char *gen_flag_string(const buffer * const b) {
 
 	static char string[MAX_FLAG_STRING_SIZE];
@@ -575,11 +562,12 @@ char *gen_flag_string(const buffer * const b) {
 	string[i++] = b->opt.del_tabs       ? 'd' : '-';
 	string[i++] = b->opt.binary         ? 'B' : ((line_desc *)b->line_desc_list.tail_pred)->line_len ? '!' : '-';
 	string[i++] = b->marking            ? (b->mark_is_vertical ? 'V' :'M') : '-';
-	string[i++] = b->recording          ? 'R' : '-';
+	string[i++] = recording_macro       ? 'R' : '-';
 	string[i++] = b->opt.preserve_cr    ? 'P' : '-';
 	string[i++] = b->is_CRLF            ? 'C' : '-';
 	string[i++] = io_utf8               ? '@' : '-';
 	string[i++] = b->encoding != ENC_8_BIT? (b->encoding == ENC_UTF8 ? 'U' : 'A') : '8';
+	mod_flag_index = i;
 	string[i++] = b->is_modified        ? '*' : '-';
 
 	if (b->opt.hex_code && !fast_gui) {
@@ -677,6 +665,12 @@ void draw_status_bar(void) {
 			strcpy(flag_string, p);
 			move_cursor(ne_lines - 1, offset + 31);
 			output_string(flag_string, true);
+			if (buffer_file_modified(cur_buffer, NULL) && !fast_gui && underline_ok) {
+				underline_on();
+				move_cursor(ne_lines - 1, offset + 31 + mod_flag_index);
+				output_char(p[mod_flag_index], -1, false);
+				underline_off();
+			}
 		}
 
 		if (!fast_gui && standout_ok) standout_off();
@@ -723,6 +717,16 @@ void draw_status_bar(void) {
 			standout_off();
 		}
 		else clear_to_eol();
+
+		if (ne_columns > 55 && buffer_file_modified(cur_buffer, NULL) && !fast_gui && underline_ok) {
+			move_cursor(ne_lines - 1, 54);
+			standout_on();
+			underline_on();
+			output_char(flag_string[mod_flag_index], -1, false);
+			underline_off();
+			standout_off();
+		}
+
 	}
 	else if (bar_gone) {
 		move_cursor(ne_lines - 1, 0);
@@ -928,7 +932,7 @@ static void error_in_menu_configuration(const int line, const char * const s) {
 }
 
 
-static void get_menu_conf(const char * menu_conf_name, char * (exists_prefs_func)()) {
+static void get_menu_conf(const char * menu_conf_name, char * (exists_prefs_func)(), config_source source) {
 	if (!menu_conf_name) menu_conf_name = MENU_CONF_NAME;
 
 	menu *new_menus = NULL;
@@ -1035,92 +1039,8 @@ static void get_menu_conf(const char * menu_conf_name, char * (exists_prefs_func
 /* Menu configs are all or nothing, so if the user has one,
    skip any global one. */
 void get_menu_configuration(const char * menu_conf_name) {
-	get_menu_conf(menu_conf_name, exists_prefs_dir);
-	if (menus == def_menus) get_menu_conf(menu_conf_name, exists_gprefs_dir);
-}
-
-
-static void error_in_key_bindings(const int line, const char * const s) {
-	fprintf(stderr, "Error in key bindings file at line %d: %s\n", line, s);
-	exit(0);
-}
-
-static void get_key_bind(const char * key_bindings_name, char * (exists_prefs_func)()) {
-	if (!key_bindings_name) key_bindings_name = KEY_BINDINGS_NAME;
-
-	char * const prefs_dir = exists_prefs_func();
-	if (prefs_dir) {
-		char * const key_bindings = malloc(strlen(prefs_dir) + strlen(key_bindings_name) + 1);
-		if (key_bindings) {
-			strcat(strcpy(key_bindings, prefs_dir), key_bindings_name);
-			char_stream * const cs = load_stream(NULL, key_bindings, false, false);
-			if (cs) {
-				char * p = cs->stream;
-				int line = 1;
-
-				while(p - cs->stream < cs->len) {
-					if (*p && !cmdcmp(KEY_KEYWORD, p)) {
-						while(*p && !isasciispace(*p)) p++;
-
-						int c;
-						if (sscanf(p, "%x %*s", &c) == 1) {
-							if (c >= 0 && c < NUM_KEYS) {
-								if (c != 27 && c != 13) {
-									while(isasciispace(*p)) p++;
-									while(*p && !isasciispace(*p)) p++;
-									while(isasciispace(*p)) p++;
-									if (*p) key_binding[c] = p;
-									else error_in_key_bindings(line, "no command specified.");
-								}
-								else error_in_key_bindings(line, "you cannot redefine ESCAPE and RETURN.");
-							}
-							else error_in_key_bindings(line, "key code out of range.");
-						}
-						else error_in_key_bindings(line, "can't read key code.");
-					}
-					else if (*p && !cmdcmp(SEQ_KEYWORD, p)) {
-						char *buf;
-						while(*p && !isasciispace(*p)) p++;	 /* skip past SEQ */
-						while(isasciispace(*p)) p++;			  /* skip to quoted sequence, like  "\x1b[A" */
-						buf = p;	/* Risky: we're replacing the double-quoted string with its parsed equivalent in situ. */
-						if (parse_string((unsigned char **)&p, (unsigned char *)buf, strlen(p)) > 0) {  /* parse_string() expects double-quoted string. */
-							while(*p && isasciispace(*p)) p++;  /* skip to key code */
-							int c;
-							if (*p && sscanf(p, "%x %*s", &c) == 1) {	 /* convert key code */
-								if (c >= 0 && c < NUM_KEYS) {
-									if (c != 27 && c != 13) {
-										if (key_may_set(buf, -c - 1) == 0)
-											error_in_key_bindings(line, "sequence table full." );
-									}
-									else error_in_key_bindings(line, "you cannot redefine ESCAPE and RETURN.");
-								}
-								else error_in_key_bindings(line, "key code out of range.");
-							}
-							else error_in_key_bindings(line, "can't read key code.");
-						}
-						else error_in_key_bindings(line, "can't read double quoted character sequence.");
-					}
-					line++;
-					p += strlen(p) + 1;
-				}
-			}
-
-			free(key_bindings);
-		}
-	}
-}
-
-char *cur_dir(void) {
-	static char *cur_dir = "./";
-	return cur_dir;
-}
-
-/* Key bindings override easily, so pull in any global bindings
-   first, then override with the users bindings. */
-void get_key_bindings(const char * key_bindings_name) {
-	get_key_bind(key_bindings_name, exists_gprefs_dir);
-	get_key_bind(key_bindings_name, exists_prefs_dir);
-	get_key_bind(key_bindings_name, cur_dir);
+	get_menu_conf(menu_conf_name, exists_prefs_dir, USER_PREFS);
+	if (menus == def_menus) get_menu_conf(menu_conf_name, exists_gprefs_dir, GLOBAL_PREFS);
 }
 
 
