@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -34,9 +34,9 @@
    when ne is called without any specific file name or macro to execute. The
    message disappears as soon as any key is typed. */
 
-char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
+char *NO_WARRANTY_msg[] = {   PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify it under",
 										"the terms of the GNU General Public License as published by the Free Software",
@@ -63,16 +63,16 @@ char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 
 char ARG_HELP[] = ABOUT_MSG "\n"
 						"Usage: ne [options] [files]\n"
-						"--help        print this message.\n"
+						"--help        print this message. [-h]\n"
 						"--           *next token is a filename.\n"
 						"+[N[,M]]     *move to last or N-th line, first or M-th column of next named file.\n"
 						"--binary     *load the next file in binary mode.\n"
-						"--read-only  *load the next file in read-only mode.\n"
+						"--read-only  *load the next file in read-only mode. [--readonly|--ro]\n"
 						"--utf8        use UTF-8 I/O.\n"
 						"--no-utf8     do not use UTF-8 I/O.\n"
 						"--ansi        use built-in ANSI control sequences.\n"
-						"--no-ansi     do not use built-in ANSI control sequences.\n"
-						"--no-config   do not read configuration files.\n"
+						"--no-ansi     do not use built-in ANSI control sequences. [--noansi]\n"
+						"--no-config   do not read configuration files. [--noconfig]\n"
 						"--no-syntax   disable syntax-highlighting support.\n"
 						"--prefs EXT   set autoprefs for the provided extension before loading the first file.\n"
 						"--keys FILE   use this file for keyboard configuration.\n"
@@ -90,8 +90,8 @@ char ARG_HELP[] = ABOUT_MSG "\n"
 
 list buffers = { (node *)&buffers.tail, NULL, (node *)&buffers.head };
 list clips = { (node *)&clips.tail, NULL, (node *)&clips.head };
-list macros = { (node *)&macros.tail, NULL, (node *)&macros.head };
-
+char_stream *recording_macro;
+bool executing_macro;
 /* global prefs, only saved in ~/.ne/.default#ap if their
    current settings differ from these defaults. Make sure these
    defaults match the conditionals in prefs.c:save_prefs(). */
@@ -194,6 +194,11 @@ int main(int argc, char **argv) {
 
 	for(int i = 1; i < argc; i++) {
 
+		if (argv[i][0] == '-' && (!strcmp(&argv[i][1], "h") || !strcmp(&argv[i][1], "-help" "\0" VERSION_STRING))) {
+			puts(ARG_HELP);
+			exit(0);
+		}
+
 		/* Special arguments start with two dashes. If we find one, we
 		   cancel its entry in argv[], so that it will be skipped when opening
 		   the specified files. The only exception is +N for skipping to the
@@ -201,10 +206,6 @@ int main(int argc, char **argv) {
 
 		if (argv[i][0] == '-' && argv[i][1] == '-') {
 			if (!argv[i][2]) i++; /* You can use "--" to force the next token to be a filename */
-			else if (!strcmp(&argv[i][2], "help" "\0" VERSION_STRING)) {
-				puts(ARG_HELP);
-				exit(0);
-			}
 			else if (!strcmp(&argv[i][2], "noconfig") || !strcmp(&argv[i][2], "no-config")) {
 				no_config = true;
 				skiplist[i] = 1; /* argv[i] = NULL; */
@@ -257,10 +258,19 @@ int main(int argc, char **argv) {
 	}
 
 #ifdef NE_TEST
-	/* Dump the builtin menu and key bindings to compare to
-	   doc/default.menus and doc/default.keys. */
-	int dump_config(void);
-	dump_config();
+	{
+		/* Dump the builtin menu and key bindings to compare to
+		   doc/default.menus and doc/default.keys. */
+		int dump_menu_config(FILE *f);
+		int dump_key_config(FILE *f);
+		FILE *f;
+
+		if ((f = fopen("ne_test_dump_default_config", "w")) ) {
+			dump_menu_config(f);
+			dump_key_config(f);
+			fclose(f);
+		}
+	}
 #endif
 
 	/* Unless --noconfig was specified, we try to configure the
@@ -271,19 +281,43 @@ int main(int argc, char **argv) {
 		get_key_bindings(key_bindings_name);
 	}
 
+#ifdef NE_TEST
+	{
+		/* Dump the builtin menu and key bindings to compare to
+		   doc/default.menus and doc/default.keys. */
+		int dump_menu_config(FILE *f);
+		int dump_key_config(FILE *f);
+		FILE *f;
+
+		if ((f = fopen("ne_test_dump_loaded_config", "w")) ) {
+			dump_menu_config(f);
+			dump_key_config(f);
+			fclose(f);
+		}
+	}
+#endif
+
 	/* If we cannot even create a buffer, better go... */
 
 	if (!new_buffer()) exit(1);
 
-	/* Now that key_bindings are loaded, try to fix up the message for NOT_FOUND. */
+	/* Now that key_bindings are loaded, try to fix up the
+	   NOT_FOUND error_msg and the LONG_INPUT_HELP info_msg. */
 	{
-		char *repeat_last_keystroke, *new_not_found;
-		if ((repeat_last_keystroke = find_key_strokes(REPEATLAST_A, 1))) {
-			if ((new_not_found = malloc(39+strlen(repeat_last_keystroke)))) {
-				strcat(strcat(strcpy(new_not_found, "Not Found. (RepeatLast with "), repeat_last_keystroke), " to wrap.)");
-				error_msg[NOT_FOUND] = new_not_found;
+		char *keystroke_string, *new_msg_text;
+		if ((keystroke_string = find_key_strokes(REPEATLAST_A, 1))) {
+			if ((new_msg_text = malloc(39+strlen(keystroke_string)))) {
+				strcat(strcat(strcpy(new_msg_text, "Not Found. (RepeatLast with "), keystroke_string), " to wrap.)");
+				error_msg[NOT_FOUND] = new_msg_text;
 			}
-			free(repeat_last_keystroke);
+			free(keystroke_string);
+		}
+		if ((keystroke_string = find_key_strokes(FIND_A, 1))) {
+			if ((new_msg_text = malloc(24+strlen(keystroke_string)))) {
+				strcat(strcat(strcpy(new_msg_text, " (browse history with "), keystroke_string), ")");
+				info_msg[LONG_INPUT_HELP] = new_msg_text;
+			}
+			free(keystroke_string);
 		}
 	}
 
@@ -431,6 +465,10 @@ int main(int argc, char **argv) {
 
 	/* We delay updates. In this way the macro activity does not cause display activity. */
 
+	#ifndef NE_TERMCAP
+	if (ansi)
+	#endif
+	ttysize();
 	reset_window();
 	delay_update();
 
@@ -446,6 +484,7 @@ int main(int argc, char **argv) {
 		   window now */
 		if (!displaying_info) {
 			refresh_window(cur_buffer);
+			if (!cur_buffer->visible_mark.shown) highlight_mark(cur_buffer, true);
 			if (cur_buffer->opt.automatch) automatch_bracket(cur_buffer, true);
 		}
 
@@ -457,7 +496,8 @@ int main(int argc, char **argv) {
 		if (window_changed_size) {
 			print_error(do_action(cur_buffer, REFRESH_A, 0, NULL));
 			window_changed_size = displaying_info = false;
-			cur_buffer->automatch.shown = 0;
+			cur_buffer->automatch.shown = false;
+			cur_buffer->visible_mark.shown = false;
 		}
 
 		if (c == INVALID_CHAR) continue; /* Window resizing. */
