@@ -1,7 +1,7 @@
 /* Main typedefs and defines.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -40,7 +40,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifndef TERMCAP
+#ifndef NE_TERMCAP
 #include <curses.h>
 #include <term.h>
 #else
@@ -78,6 +78,12 @@ typedef enum {
 
 extern const char *input_class_names[];
 
+typedef enum {
+	BUILT_IN, GLOBAL_PREFS, USER_PREFS, CUR_DIR, CONFIG_SOURCE_COUNT
+} config_source;
+
+extern const char *config_source_names[];
+
 /* This is the expected max length of the current directory name. */
 
 #define CUR_DIR_MAX_SIZE    (16*1024)
@@ -163,18 +169,21 @@ typedef struct {
 		allow_dupes:1,     /* Searches are more efficient if we have no duplicate entries. */
 		allow_reorder:1,   /* Allow NextDoc/PrevDoc keys to re-order entries. */
 		ignore_tab:1,      /* Permits Tab to exit requester. */
-		reordered:1;       /* Indicates whether reordering was done during request. */
+		reordered:1,       /* Indicates whether reordering was done during request. */
+		prune:1,           /* Whether to start off pruning by partial input. */
+		find_quits:1,      /* Map FIND_A to QUIT_A (for long input ^F requester). */
+		help_quits:1,      /* Map HELP_A to QUIT_A (for help requester). */
+		selectdoc_quits:1; /* Map SELECTDOC_A to QUIT_A (for F4 requester). */
 	char suffix;
-	
-	int cur_entries;
-	int alloc_entries;
-	int max_entry_len;
-	char **entries;
-	int *orig_order;      /* maps from current order to original order when allow_reorder is true. */
-	
-	int cur_chars;
-	int alloc_chars;
-	char *chars;
+	int cur_entries;      /* count of entries */
+	int alloc_entries;    /* allocated slots in **entries */
+	char **entries;       /* The array of alloc_entries string pointers pointing into *chars */
+	int *lengths;         /* When set, the column width needed to display corresponding entry (strlen() + suffix + blank) */
+	int *reorder;         /* maps from original order to new order when allow_reorder is true. */
+
+	int cur_chars;        /* count of used characters */
+	int alloc_chars;      /* allocated characters in *chars */
+	char *chars;          /* contiguous block of allocated characters */
 } req_list;
 
 /* These are the list and node structures used throughout ne. See the exec.c
@@ -227,7 +236,7 @@ typedef struct {
 
 /* This structure defines a line descriptor; it is a node containing a pointer
    to the line text, and an integer containing the line length in bytes. The
-   line pointer by ld_text is NOT NULL-terminated. line_len is zero iff line
+   line pointed to line is NOT NULL-terminated. line_len is zero iff line
    is NULL, in which case we are representing an empty line. ld_node->next
    is NULL iff this node is free for use. */
 
@@ -434,7 +443,7 @@ typedef struct {
 		binary:1,          /* Load and save in binary mode */
 		utf8auto:1,        /* Try to detect automatically UTF-8 */
 		visual_bell:1;     /* Prefer visible bell to audible */
-  } options_t;
+} options_t;
 
 #ifndef NDEBUG
 #define assert_options(o) {if ((o)) {\
@@ -480,10 +489,10 @@ typedef struct {
 	int64_t num_lines;
 	int64_t block_start_line, block_start_pos;
 	struct {
-		int shown;
-		int x;                 /* Visual (on-screen) coordinates of the highlighted bracket, if shown is true. */
-		int y;
-	} automatch;
+		bool shown;
+		int x;                 /* screen x coordinate of the highlighted bracket or mark if shown is true. */
+		int y;                 /* screen y coordinate of the highlighted bracket or mark if shown is true. */
+	} automatch, visible_mark;
 	int64_t allocated_chars;
 	int64_t free_chars;
 	encoding_type encoding;
@@ -497,7 +506,7 @@ typedef struct {
 	int cur_bookmark;           /* For Goto(Next|Prev)Bookmark. */
 
 	struct high_syntax *syn;    /* Syntax loaded for this buffer. */
-	uint32_t *attr_buf;              /* If attr_len >= 0, a pointer to the list of *current* attributes of the *current* line. */ 
+	uint32_t *attr_buf;              /* If attr_len >= 0, a pointer to the list of *current* attributes of the *current* line. */
 	int64_t attr_size;              /* attr_buf size. */
 	int64_t attr_len;               /* attr_buf valid number of characters, or -1 to denote that attr_buf is not valid. */
 	HIGHLIGHT_STATE next_state; /* If attr_len >= 0, the state after the *current* line. */
@@ -506,7 +515,6 @@ typedef struct {
 
 	unsigned int
 		is_modified:1,           /* Buffer has been modified since last save */
-		recording:1,             /* We are recording a macro */
 		marking:1,               /* We are marking a block */
 		x_wanted:1,              /* We're not where we would like to be */
 		y_wanted:1,              /* We've been paging up/down */
@@ -517,8 +525,6 @@ typedef struct {
 		redoing:1,               /* We are currently redoing an action */
 		mark_is_vertical:1,      /* The current marking is vertical */
 		atomic_undo:1,           /* subsequent commands undo as a block */
-		executing_macro:1,       /* We are currently executing a macro. */
-		executing_internal_macro:1,  /* We are currently executing the internal macro of the current buffer */
 		is_CRLF:1;               /* Buffer should be saved with CR/LF terminators */
 
 	unsigned int find_string_changed; /* 0 = unset; 1 = force; else prior search's serial number */
@@ -566,11 +572,13 @@ typedef struct {
 
 extern const char *key_binding[];
 
+extern config_source key_binding_source[NUM_KEYS];
+
 extern buffer *cur_buffer;
 
 /* These are the global lists. */
 
-extern list buffers, clips, macros;
+extern list buffers, clips;
 
 /* This integer keeps the global turbo parameter. */
 
@@ -635,7 +643,7 @@ key combinations required to produce those key codes. */
 
 extern const char *key_stroke[];
 
-/* A boolean recording whether the last replace was for an empty string 
+/* A boolean recording whether the last replace was for an empty string
   (of course, this can happen only with regular expressions). */
 
 extern bool last_replace_empty_match;
@@ -646,6 +654,18 @@ extern bool last_replace_empty_match;
 
 #define MACRO_HASH_TABLE_SIZE (101)
 
+
+/* When recording a macro, this points to the recording stream.
+   When not recording a macro, this is NULL. */
+
+extern char_stream *recording_macro;
+
+
+/* Whether we're currently executing a macro. */
+
+extern bool executing_macro;
+
+
 /* There are several functions that set the status bar:
    print_message(), print_prompt() and input_refresh(), and
    draw_status_bar(). If we're suspended while a requester is up, this
