@@ -1,7 +1,7 @@
 /* Function prototypes
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -41,6 +41,8 @@ void free_buffer(buffer *b);
 int64_t calc_lost_chars(const buffer *b);
 buffer *get_nth_buffer(int n);
 buffer *get_buffer_named(const char *p);
+bool is_buffer(const buffer *b);
+bool is_buffer_empty(const buffer * const b);
 int modified_buffers(void);
 int save_all_modified_buffers(void);
 line_desc *alloc_line_desc(buffer *b);
@@ -88,7 +90,7 @@ int execute_command_line(buffer *b, const char *command_line);
 macro_desc *alloc_macro_desc(void);
 void free_macro_desc(macro_desc *md);
 void record_action(char_stream *cs, action a, int64_t c, const char *p, bool verbose);
-int play_macro(buffer *b, char_stream *cs);
+int play_macro(char_stream *cs);
 macro_desc *load_macro(const char *name);
 int execute_macro(buffer *b, const char *name);
 void help(char *p);
@@ -96,6 +98,8 @@ int cmdcmp(const char *c, const char *m);
 void unload_macros(void);
 char *find_key_strokes(int c, int n);
 void optimize_macro(char_stream *cs, bool verbose);
+int parse_word_parm(char *p, char *pat, int64_t *match);
+
 
 /* display.c */
 void update_syntax_states(buffer *b, int row, line_desc *ld, line_desc *end_ld);
@@ -115,6 +119,7 @@ void scroll_window(buffer *b, line_desc *ld, int line, int n);
 void ensure_attributes(buffer *b);
 void store_attributes(buffer *b, line_desc *ld);
 void automatch_bracket(buffer * const b, const bool show);
+void highlight_mark(buffer * const b, const bool show);
 
 /* edit.c */
 int to_upper(buffer *b);
@@ -123,7 +128,7 @@ int capitalize(buffer *b);
 int match_bracket(buffer *b);
 int find_matching_bracket(buffer *b, const int64_t min_line, const int64_t max_line, int64_t *match_line, int64_t *match_pos, int *c, line_desc ** ld);
 int64_t word_wrap(buffer *b);
-int paragraph(buffer *b);
+int paragraph(buffer * const b, const bool mark_for_undo);
 int center(buffer *b);
 int auto_indent_line(buffer * const b, const int64_t line, line_desc * const ld, const int64_t up_to_col);
 int backtab(buffer *b);
@@ -151,7 +156,8 @@ const char *ext2syntax(const char * const ext);
 void read_key_capabilities(void);
 void set_escape_time(int new_escape_time);
 int get_key_code(void);
-int key_may_set(const char * const cap_string, int code);
+int key_may_set(const char * const cap_string, int code, config_source source);
+void get_key_bindings(const char *);
 
 /* menu.c */
 void print_message(const char *message);
@@ -165,7 +171,6 @@ void print_info(int info_num);
 void alert(void);
 void handle_menus(void);
 void get_menu_configuration(const char *);
-void get_key_bindings(const char *);
 
 /* names.c */
 
@@ -194,8 +199,9 @@ void reset_position_to_sof(buffer *b);
 void resync_pos(buffer * const b);
 void toggle_sof_eof(buffer *b);
 void toggle_sol_eol(buffer *b);
-int  search_word(buffer *b, int dir);
+int  search_word(buffer *b, int dir, bool start);
 void move_to_eow(buffer *b);
+void move_to_sow(buffer *b);
 void move_inc_down(buffer *b);
 void move_inc_up(buffer *b);
 
@@ -225,6 +231,7 @@ bool  request_response(const buffer *b, const char *prompt, bool default_value);
 char  request_char(const buffer *b, const char *prompt, const char default_value);
 int64_t request_number(const buffer *b, const char *prompt, int64_t default_value);
 char *request_string(const buffer *b, const char *prompt, const char *default_string, bool accept_null_string, int completion_type, bool prefer_utf8);
+/* char *complete_filename(const char *start_prefix); */
 char *request(const buffer *b, const char *prompt, const char *default_string, bool alpha_allowed, int completion_type, bool prefer_utf8);
 
 /* request.c */
@@ -233,7 +240,6 @@ char *request_syntax();
 char *request_files(const char *filename, bool use_prefix);
 char *request_file(const buffer *b, const char *prompt, const char *default_name);
 int   request_document(void);
-char *complete_filename(const char *start_prefix);
 int   req_list_del(req_list * const rl, int nth);
 void  req_list_free(req_list * const rl);
 int   req_list_init(req_list * const rl, int cmpfnc(const char *, const char *), const bool allow_dupes, const bool allow_reorder, const char suffix);
@@ -262,6 +268,7 @@ void handle_winch(int sig);
 char_stream *alloc_char_stream(int64_t size);
 void free_char_stream(char_stream *cs);
 char_stream *realloc_char_stream(char_stream *cs, int64_t size);
+char_stream *dup_stream(char_stream * const cs);
 int add_to_stream(char_stream *cs, const char *s, int64_t len);
 char_stream *reset_stream(char_stream *cs);
 void set_stream_encoding(char_stream *cs, encoding_type source);
@@ -285,6 +292,7 @@ unsigned long file_mod_time(const char *filename);
 ssize_t read_safely(const int fd, void * const buf, const int64_t len);
 bool buffer_file_modified(const buffer *b, const char *name);
 char *str_dup(const char *s);
+char *strntmp(const char * const s, const int len);
 int64_t strnlen_ne(const char *s, int64_t n);
 int strcmpp(const void *a, const void *b);
 int strdictcmpp(const void *a, const void *b);
