@@ -1,7 +1,7 @@
 /* Search/replace functions (with and without regular expressions).
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -48,9 +48,9 @@ static unsigned int d[256];
 static unsigned int search_serial_num = 2;
 
 /* This macro upper cases a character or not, depending on the boolean
-sense_case. It is used in find(). Note that the argument *MUST* be unsigned. */
+sense_case. It is used in find(). The result is always unsigned. */
 
-#define CONV(c) (sense_case ? c : up_case[c])
+#define CONV(c) (sense_case ? (unsigned char)c : up_case[(unsigned char)c])
 
 
 /* This vector is a translation table for the regex library which maps
@@ -122,12 +122,12 @@ int find(buffer * const b, const char *pattern, const bool skip_first, bool wrap
 	if (! b->opt.search_back) {
 
 		if (recompile_string) {
-			for(int i = 0; i < m - 1; i++) d[CONV((unsigned char)pattern[i])] = m - i-1;
+			for(int i = 0; i < m - 1; i++) d[CONV(pattern[i])] = m - i-1;
 			b->find_string_changed = search_serial_num;
 		}
 
 		char * p = ld->line + b->cur_pos + m - 1 + (skip_first ? 1 : 0);
-		const unsigned char first_char = CONV((unsigned char)pattern[m - 1]);
+		const unsigned char first_char = CONV(pattern[m - 1]);
 		int64_t wrap_lines_left = b->num_lines + 1;
 
 		while(y < b->num_lines && !stop && wrap_lines_left--) {
@@ -137,12 +137,12 @@ int find(buffer * const b, const char *pattern, const bool skip_first, bool wrap
 			if (ld->line_len >= m) {
 
 				while((p - ld->line) < ld->line_len) {
-					const unsigned char c = CONV((unsigned char)*p);
+					const unsigned char c = CONV(*p);
 					if (c != first_char) p += d[c];
 					else {
 						int i;
 						for (i = 1; i < m; i++)
-							if (CONV((unsigned char)*(p - i)) != CONV((unsigned char)pattern[m - i-1])) {
+							if (CONV(*(p - i)) != CONV(pattern[m - i-1])) {
 								p += d[c];
 								break;
 							}
@@ -168,12 +168,12 @@ int find(buffer * const b, const char *pattern, const bool skip_first, bool wrap
 	else {
 
 		if (recompile_string) {
-			for(int i = m - 1; i > 0; i--) d[CONV((unsigned char)pattern[i])] = i;
+			for(int i = m - 1; i > 0; i--) d[CONV(pattern[i])] = i;
 			b->find_string_changed = search_serial_num;
 		}
 
 		char * p = ld->line + (b->cur_pos > ld->line_len - m ? ld->line_len - m : b->cur_pos + (skip_first ? -1 : 0));
-		const unsigned char first_char = CONV((unsigned char)pattern[0]);
+		const unsigned char first_char = CONV(pattern[0]);
 		int64_t wrap_lines_left = b->num_lines + 1;
 
 		while(y >= 0 && !stop && wrap_lines_left--) {
@@ -184,12 +184,12 @@ int find(buffer * const b, const char *pattern, const bool skip_first, bool wrap
 
 				while((p - ld->line) >= 0) {
 
-					const unsigned char c = CONV((unsigned char)*p);
+					const unsigned char c = CONV(*p);
 					if (c != first_char) p -= d[c];
 					else {
 						int i;
 						for (i = 1; i < m; i++)
-							if (CONV((unsigned char)*(p + i)) != CONV((unsigned char)pattern[i])) {
+							if (CONV(*(p + i)) != CONV(pattern[i])) {
 								p -= d[c];
 								break;
 							}
@@ -277,7 +277,6 @@ static struct re_registers re_reg;
    RE_NREGS, in which case there is no way to recover it. */
 
 static int map_group[RE_NREGS];
-static int use_map_group;
 
 /* Works exactly like find(), but uses the regex library instead. */
 
@@ -331,7 +330,7 @@ int find_regexp(buffer * const b, const char *regex, const bool skip_first, bool
 			const char *s;
 			char *q;
 			bool escape = false;
-			int virtual_group = 0, real_group = 0, dots = 0, comps = 0, nonwords = 0, use_map_group = 0;
+			int virtual_group = 0, real_group = 0, dots = 0, comps = 0, nonwords = 0;
 
 			s = regex;
 
@@ -390,10 +389,7 @@ int find_regexp(buffer * const b, const char *regex, const bool skip_first, bool
 					}
 					else if (*s == '(') {
 						*(q++) = '(';
-						if (virtual_group < RE_NREGS - 1) {
-							map_group[++virtual_group] = ++real_group;
-							use_map_group = virtual_group;
-						}
+						if (virtual_group < RE_NREGS - 1) map_group[++virtual_group] = ++real_group;
 					}
 					else if (*s == '[') {
 						if (*(s+1) == '^') {
@@ -500,17 +496,17 @@ int find_regexp(buffer * const b, const char *regex, const bool skip_first, bool
 
 /* This allows regexp users to retrieve matched substrings.
    They are responsible for freeing these strings.
-   i0 should be <= number of paren groups in original regex. */
-char *nth_regex_substring(const line_desc *ld, int i0) {
+   n should be <= number of paren groups in original regex.
+   Note that n is the actual group index--no remapping is performed. */
+char *nth_regex_substring(const line_desc *ld, const int n) {
 	char *str;
-	int i;
 
-	if ((i = use_map_group ? map_group[i0] : i0) >= RE_NREGS) return NULL;
+	if (n >= RE_NREGS) return NULL;
 
-	if (i > 0 && i < re_reg.num_regs ) {
-		if (str = malloc(re_reg.end[i] - re_reg.start[i] + 1)) {
-			memcpy(str, ld->line + re_reg.start[i], re_reg.end[i] - re_reg.start[i]);
-			str[re_reg.end[i] - re_reg.start[i]] = 0;
+	if (n > 0 && n < re_reg.num_regs ) {
+		if (str = malloc(re_reg.end[n] - re_reg.start[n] + 1)) {
+			memcpy(str, ld->line + re_reg.start[n], re_reg.end[n] - re_reg.start[n]);
+			str[re_reg.end[n] - re_reg.start[n]] = 0;
 			return str;
 		}
 	}
@@ -518,11 +514,12 @@ char *nth_regex_substring(const line_desc *ld, int i0) {
 }
 
 
-/* This allows regexp users to check whether matched substrings are nonempty. */
-bool nth_regex_substring_nonempty(const line_desc *ld, int i0) {
-	int i;
-	if ((i = use_map_group ? map_group[i0] : i0) >= RE_NREGS) return false;
-	if (i > 0 && i < re_reg.num_regs) return re_reg.start[i] != re_reg.end[i];
+/* This allows regexp users to check whether matched substrings are nonempty.
+   Note that n is the actual group index--no remapping is performed. */
+bool nth_regex_substring_nonempty(const line_desc *ld, const int n) {
+	if (n >= RE_NREGS) return false;
+
+	if (n > 0 && n < re_reg.num_regs) return re_reg.start[n] != re_reg.end[n];
 	return false;
 }
 
