@@ -1,7 +1,7 @@
 /* Miscellaneous support functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -88,35 +88,42 @@ void normalize_path(char *c) {
    The returned string has at least one extra char so it can be shifted
    if necessary as per relative_file_path(). */
 
-char *absolute_file_path(const char *a, const char *b) {
-	char *c = malloc(strlen(a)+strlen(b)+2);
+char *absolute_file_path(const char *a0, const char *b) {
+	char *a, *c;
 	char *cc;
 	const char *aa;
-	if (!a || !b || !c || a[0] == '/' || b[0] != '/') {
-		if (c) free(c);
-		return NULL;
-	}
-	strcpy(c, b);
-	normalize_path(c);
-	cc = c+strlen(c);
-	aa = a;
-	while (*aa) {
-		if (aa[0] == '.' && aa[1] == '.' && aa[2] == '/') {
-			aa += 3;
-			while (cc > c && *--cc != '/')
-				;
-		} else if (aa[0] == '.' && aa[1] == '/') {
-			aa += 2;
-		} else {
-			*cc++ = '/';
-			do {
-				*cc++ = *aa++;
-			} while (*aa && *aa != '/');
-			if (*aa == '/') aa++;
+	if (!a0 || !b || b[0] != '/') return NULL;
+
+	/* If *a is already an absolute path, get its relative equivalent first. */
+	if (a0[0] == '/') {
+		a = relative_file_path(a0, b);
+		if (!a) return NULL;
+	} else a = (char *)a0;
+
+	if ((c = malloc(strlen(a) + strlen(b) + 2))) {
+		strcpy(c, b);
+		normalize_path(c);
+		cc = c + strlen(c);
+		aa = a;
+		while (*aa) {
+			if (aa[0] == '.' && aa[1] == '.' && aa[2] == '/') {
+				aa += 3;
+				while (cc > c && *--cc != '/')
+					;
+			} else if (aa[0] == '.' && aa[1] == '/') {
+				aa += 2;
+			} else {
+				*cc++ = '/';
+				do {
+					*cc++ = *aa++;
+				} while (*aa && *aa != '/');
+				if (*aa == '/') aa++;
+			}
 		}
+		*cc = '\0';
+		normalize_path(c);
 	}
-	*cc = '\0';
-	normalize_path(c);
+	if (a != a0) free (a);
 	return c;
 }
 
@@ -127,39 +134,40 @@ char *absolute_file_path(const char *a, const char *b) {
    The returned string has one extra '\0' so request_files() can shift it. */
 
 char *relative_file_path(const char *aa, const char *b) {
-	int common_dirs=0, up_dirs=0, i, j=0;
+	int up_dirs=0, i, j=0;
 	char *a, *c;
 	if (!aa || !b) return NULL;
+
 	a = str_dup(aa);
 	if (!a) return NULL;
 	normalize_path(a);
+
 	int match = max_prefix(a, b);
 	if (a[0] != '/' || b[0] != '/') {
 		if (a) free(a);
 		return NULL;
 	}
-	for (i=1; i<match; i++) {  /* skip initial '/' */
+
+	for (i=0; i<=match; i++) {
 		if (a[i] == '/') {
-			common_dirs++;
-			j = i + 1;
-		} else if (b[i+1] == '\0') {
-			common_dirs++;
-			j = i + 2;
+			j = i;
 		}
 	}
 
-	for (i=j; i<=strlen(b); i++) {
-		if (b[i] == '/' || b[i] == '\0') up_dirs++;
+	for (i=j; i<strlen(b); i++) {
+		if (b[i] == '/') up_dirs++;
 	}
+
 	int newlen = 3 * up_dirs + (strlen(a+j) ) + 2; /* 3 for each "../" and two trailing '\0' */
 	c = malloc(newlen);
 	if (c) {
 		*c = '\0';
 		for (i=0; i<up_dirs; i++)
 			strcat(c, "../");
-		strcat(c, a + j);
+		strcat(c, a + j + 1);
 		normalize_path(c);
 	}
+
 	return c;
 }
 
@@ -356,6 +364,37 @@ char *str_dup(const char * const s) {
 	return dup;
 }
 
+/* Copies len chars from s to a buffer, terminated with '\0'.
+   The buffer is reused by strntmp(), so the caller must complete its
+   use of the returned string before the next call to strntmp().
+   If len < 0, the buffer is freed.
+   Returns the address of the buffer. */
+char *strntmp(const char * const s, const int len) {
+	static char *buf = NULL;
+	static uint64_t buflen = 0;
+	char *buf_new;
+	int cplen = len;
+
+	if (len < 1) {
+		if (buf) free(buf);
+		buf = NULL;
+		buflen = 0;
+	} else {
+		if (len >= buflen) {
+			if (buf_new = realloc(buf, len * 2 + 1)) {
+				buflen = len * 2 + 1;
+				buf = buf_new;
+			}
+			else if (buflen)
+				cplen = buflen - 1;
+			else cplen = 0;
+		}
+		if (cplen) strncpy(buf, s, cplen);
+		buf[cplen] = '\0';
+	}
+	return buf;
+}
+
 /* Tries to compute the length as a string of the given pointer,
    but stops after n characters (returning n). */
 
