@@ -1,7 +1,7 @@
 /* Miscellaneous inline support functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -215,5 +215,5 @@ static bool inline ne_isspace(const int c, const int encoding) {
    For now, hardcode '_'.  */
 
 static bool inline ne_isword(const int c, const int encoding) {
-	return c == '_' || !(ne_isspace(c, encoding) || ne_ispunct(c, encoding));
+	return c == '_' || !(c == '\0' || ne_isspace(c, encoding) || ne_ispunct(c, encoding));
 }
