@@ -1,7 +1,7 @@
 /* extern's of flags describing terminal's characteristics.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2021 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -94,6 +94,8 @@ extern bool scroll_region_ok;   /* Terminal supports setting the scroll window *
 
 extern bool standout_ok;        /* Terminal supports standout without magic cookies */
 
+extern bool underline_ok;       /* Terminal supports underline without magic cookies */
+
 extern bool cursor_on_off_ok;   /* Terminal can make the cursor visible or invisible */
 
 extern bool ansi_color_ok;      /* Terminal supports ANSI color */
@@ -195,3 +197,5 @@ extern char *ne_clr_eol;
 extern bool ne_transparent_underline;
 
 extern bool io_utf8;
+
+extern uint32_t curr_attr;
